@@ -14,11 +14,17 @@ import traceback
 from logging import debug, error
 from typing import Callable
 
+from portmod.download import (
+    RemoteHashError,
+    SourceUnfetchable,
+    download_source,
+    fetchable,
+)
 from portmod.globals import env
 from portmod.loader import SandboxedError, load_file
 from portmod.news import validate_news
 from portmod.pybuild import Pybuild
-from portmod.repo import Repo, get_repo_name, get_repo_root
+from portmod.repo import Repo, get_repo, get_repo_name, get_repo_root, has_repo
 from portmod.repo.loader import get_atom_from_path
 from portmod.repo.metadata import (
     get_categories,
@@ -58,7 +64,6 @@ def scan_package_dir(path: str, err: Callable[[str], None]):
         except SandboxedError as e:
             err(f"{e}")
         except Exception as e:
-            traceback.print_exc()
             err(f"{e}")
 
 
@@ -351,12 +356,24 @@ def commit_message(args, repo_root: str, err: Callable[[str], None]):
             print(message)
 
 
+def scan_sources(file: str, err: Callable[[str], None]):
+    pkg = load_file(file)
+
+    for source in fetchable(pkg, matchall=True):
+        try:
+            download_source(pkg, source, check_remote=True)
+        except (SourceUnfetchable, RemoteHashError) as e:
+            err(f"In file {file}: {e}")
+
+
 def scan(args, repo_root: str, err: Callable[[str], None]):
     if args.diff:
         import git
 
         for file in git.Git().diff(args.diff, name_only=True).splitlines():
             scan_file(os.path.join(repo_root, file), repo_root, err)
+            if file.endswith(".pybuild"):
+                scan_sources(file, err)
 
         for commit in git.Git().log("HEAD", "^" + args.diff, pretty="%H").splitlines():
             scan_commit(commit, err)
@@ -474,8 +491,12 @@ def test_install_cli(args, repo_root, _err):
             # we can only feasibly do it if creating a prefix from scratch.
             # If using an existing prefix, we should just use the existing profile
 
+            repo_name = get_repo_name(repo_root)
+            if not has_repo(repo_name) or get_repo(repo_name).location != repo_root:
+                env.REPOS.insert(0, Repo(repo_name, repo_root))
+                clear_system_cache()
             env.set_prefix(test_prefix)
-            add_prefix_repo(get_repo_name(repo_root))
+            add_prefix_repo(repo_name)
             profiles = list_profiles(args.arch)
             print(f"Using profile {profiles[0]}")
             set_profile(profiles[0])
