@@ -18,10 +18,10 @@ from portmod.query import (
 )
 from portmod.repo.keywords import get_unstable_flag
 from portmod.repo.metadata import get_package_metadata
+from portmodlib._deprecated.vfs import find_file, list_dir
 from portmodlib.atom import Atom
 from portmodlib.colour import blue, bright, green, red
 from portmodlib.l10n import l10n
-from portmodlib.vfs import find_file, list_dir
 
 from . import atom_metavar
 
