@@ -1,7 +1,7 @@
 # Copyright 2019-2021 Portmod Authors
 # Distributed under the terms of the GNU General Public License v3
 
-import copy
+from collections import defaultdict
 from logging import warning
 from types import SimpleNamespace
 from typing import Any, Dict, Iterable, List, Optional, Sequence, Set, Tuple, Union
@@ -16,7 +16,14 @@ from portmodlib.atom import Atom, FQAtom, QualifiedAtom, atom_sat
 from portmodlib.colour import blue, green
 from portmodlib.usestr import human_readable_required_use, parse_usestr
 
-from .tokens import expand_use_conditionals, negate, token_conflicts
+from .tokens import (
+    AtomToken,
+    FlagToken,
+    Token,
+    VariableToken,
+    expand_use_conditionals,
+    token_conflicts,
+)
 
 
 class Target(SimpleNamespace):
@@ -44,35 +51,41 @@ class Formula:
 
     __i = 1  # Integer counter, for the numerical equivalent of tokens
     __j = 1  # Variable name counter.
-    __numcache: Dict[str, int] = {}  # Strings are
-    __stringcache: Dict[int, str] = {}
-    __variabledesc: Dict[str, str] = {}
+    __numcache: Dict[Token, int] = {}  # Strings are
+    __stringcache: Dict[int, Token] = {}
+    __variabledesc: Dict[Token, str] = {}
 
     class Clause:
         """Generic clause type"""
 
-        def __init__(self, clause: Iterable[str]):
-            self.requirements: Set[str] = set()
+        def __init__(self, clause: Iterable[Token], source):
+            self.requirements: Set[Token] = set()
             self.clause = clause
             self.intclause: Optional[Iterable[int]] = None
+            self.source = source
 
         def str2num(self):
             """
             Converts the tokens in the clause to integers for use with pysat
             """
-            result = copy.copy(self)
-            result.intclause = list(map(Formula.getnum, self.clause))
-            return result
+            self.intclause = list(map(Formula.getnum, self.clause))
+            return self
 
         def sourceatom(self) -> Optional[Atom]:
             return None
 
+        def colourless(self) -> str:
+            ...
+
+        def blocks(self, model: Set[Token], clause: "Formula.Clause") -> bool:
+            return False
+
     class MetaClause(Clause):
         def __init__(
             self,
             source: Optional[str],
             desc: Optional[str],
-            clause: Iterable[str],
+            clause: Iterable[Token],
             weight: Optional[int],
             atmost: Optional[int],
         ):
@@ -81,7 +94,7 @@ class Formula:
             self.clause = clause
             self.weight = weight
             self.atmost = atmost
-            self.requirements: Set[str] = set()
+            self.requirements: Set[Token] = set()
 
         def __str__(self):
             return f"{self.source} - {self.desc}"
@@ -92,15 +105,17 @@ class Formula:
             return None
 
     class DepClause(Clause):
-        def __init__(self, source: str, clause: Iterable[str], dependency: Atom):
-            super().__init__(clause)
-            self.source = source
+        def __init__(self, source: str, clause: Iterable[Token], dependency: Atom):
+            super().__init__(clause, source)
             self.dependency = dependency
 
         def __str__(self):
             return f"{green(self.dependency)}: required by {green(self.source)}"
 
-        def blocks(self, model: Set[str], clause: "Formula.MetaClause") -> bool:
+        def colourless(self):
+            return f"{self.dependency}: required by {self.source}"
+
+        def blocks(self, model: Set[Token], clause: "Formula.Clause") -> bool:
             if isinstance(clause, Formula.BlockerClause):
                 if atom_sat(self.dependency, clause.blocked) and all(
                     req in model for req in self.requirements
@@ -115,15 +130,17 @@ class Formula:
             return None
 
     class BlockerClause(Clause):
-        def __init__(self, source: str, clause: Iterable[str], blocked: Atom):
-            super().__init__(clause)
-            self.source = source
+        def __init__(self, source: str, clause: Iterable[Token], blocked: Atom):
+            super().__init__(clause, source)
             self.blocked = blocked
 
         def __str__(self):
             return f"{green(self.blocked)}: blocked by {green(self.source)}"
 
-        def blocks(self, model: Set[str], clause: "Formula.MetaClause") -> bool:
+        def colourless(self):
+            return f"{self.blocked}: blocked by {self.source}"
+
+        def blocks(self, model: Set[Token], clause: "Formula.Clause") -> bool:
             if isinstance(clause, Formula.DepClause):
                 if atom_sat(self.blocked, clause.dependency) and all(
                     req in model for req in self.requirements
@@ -139,10 +156,9 @@ class Formula:
 
     class UseDepClause(Clause):
         def __init__(
-            self, source: str, clause: Iterable[str], depatom: Atom, flag: str
+            self, source: str, clause: Iterable[Token], depatom: Atom, flag: FlagToken
         ):
-            super().__init__(clause)
-            self.source = source
+            super().__init__(clause, source)
             self.depatom = depatom
             self.flag = flag
 
@@ -151,7 +167,10 @@ class Formula:
                 f"{green(self.depatom)}[{self.flag}]: required by {green(self.source)}"
             )
 
-        def blocks(self, model: Set[str], clause: "Formula.MetaClause") -> bool:
+        def colourless(self):
+            return f"{self.depatom}[{self.flag}]: required by {self.source}"
+
+        def blocks(self, model: Set[Token], clause: "Formula.Clause") -> bool:
             if isinstance(clause, Formula.UseDepClause):
                 if (
                     atom_sat(self.depatom, clause.depatom)
@@ -173,24 +192,32 @@ class Formula:
         def __init__(
             self,
             atom: Atom,
-            clause: Iterable[str],
-            flag: str,
+            clause: Iterable[Token],
+            flag: Token,
             tokens: List[Union[str, List]],
         ):
-            super().__init__(clause)
-            self.atom = self.source = atom
+            super().__init__(clause, atom)
+            self.atom = atom
             self.flag = flag
             self.tokens = tokens
 
-        def __str__(self):
-            if self.flag.startswith("__"):  # If flag is a generated variable
+        def _get_required_use_str(self) -> Tuple[str, str]:
+            if isinstance(self.flag, VariableToken):  # If flag is a generated variable
                 string = Formula.get_variable_desc(self.flag)
-            else:
-                if self.flag[0] == "-":
-                    string = "-" + list(Atom(self.flag[1:]).USE)[0]
+            elif isinstance(self.flag, FlagToken):
+                if self.flag.polarity:
+                    string = list(self.flag.value.USE)[0]
                 else:
-                    string = list(Atom(self.flag).USE)[0]
+                    string = "-" + list(self.flag.value.USE)[0]
+            else:
+                raise Exception(
+                    f"AtomToken {self.flag} passed as the flag to RequiredUseClause!"
+                )
             parent = human_readable_required_use(self.tokens)
+            return string, parent
+
+        def __str__(self):
+            string, parent = self._get_required_use_str()
             # FIXME: Localize
             if string == parent:
                 return f"{green(self.atom)} could not satisfy {blue(string)}"
@@ -200,7 +227,17 @@ class Formula:
                     f"the larger clause {blue(parent)}"
                 )
 
-        def blocks(self, model: Set[str], clause: "Formula.MetaClause") -> bool:
+        def colourless(self):
+            string, parent = self._get_required_use_str()
+            if string == parent:
+                return f"{self.atom} could not satisfy {string}"
+            else:
+                return (
+                    f"{self.atom} could not satisfy {string}, which is part of "
+                    f"the larger clause {parent}"
+                )
+
+        def blocks(self, model: Set[Token], clause: "Formula.Clause") -> bool:
             if isinstance(clause, Formula.UseDepClause):
                 if (
                     atom_sat(self.atom, clause.depatom)
@@ -223,52 +260,53 @@ class Formula:
 
     def __init__(self):
         self.clauses = []
-        self.atoms: Dict[QualifiedAtom, Set[FQAtom]] = {}
+        self.atoms: Dict[QualifiedAtom, Set[FQAtom]] = defaultdict(set)
         self.flags: Set[FQAtom] = set()
 
     @classmethod
-    def getnum(cls, string: str) -> int:
-        if string in cls.__numcache:
-            return cls.__numcache[string]
+    def getnum(cls, token: Token) -> int:
+        if token in cls.__numcache:
+            return cls.__numcache[token]
 
-        if string[0] == "-":
-            cls.__numcache[string] = -cls.__i
-            cls.__numcache[string[1:]] = cls.__i
-            cls.__stringcache[-cls.__i] = string
-            cls.__stringcache[cls.__i] = string[1:]
+        if token.polarity:
+            cls.__numcache[token] = cls.__i
+            cls.__numcache[token.neg()] = -cls.__i
+            cls.__stringcache[cls.__i] = token
+            cls.__stringcache[-cls.__i] = token.neg()
         else:
-            cls.__numcache[string] = cls.__i
-            cls.__numcache["-" + string] = -cls.__i
-            cls.__stringcache[cls.__i] = string
-            cls.__stringcache[-cls.__i] = "-" + string
+            cls.__numcache[token] = -cls.__i
+            cls.__numcache[token.neg()] = cls.__i
+            cls.__stringcache[-cls.__i] = token
+            cls.__stringcache[cls.__i] = token.neg()
 
         cls.__i += 1
-        return cls.__numcache[string]
+        return cls.__numcache[token]
 
     @classmethod
-    def getstring(cls, num: int) -> str:
+    def getstring(cls, num: int) -> Token:
         return cls.__stringcache[num]
 
     @classmethod
-    def genvariable(cls, desc: List[Any]) -> str:
-        var = "__" + str(cls.__j)
+    def genvariable(cls, desc: List[Any]) -> Token:
+        var = VariableToken("__" + str(cls.__j))
         cls.__j += 1
         cls.__variabledesc[var] = human_readable_required_use(desc)
         return var
 
     @classmethod
-    def get_variable_desc(cls, var: str) -> str:
+    def get_variable_desc(cls, var: Token) -> str:
         return cls.__variabledesc[var]
 
     def merge(self, other: "Formula"):
         self.clauses.extend(other.clauses)
-        for clause in other.clauses:
-            self.__update_for_clause__(clause.clause)
+        self.flags |= other.flags
+        for atom, values in other.atoms.items():
+            self.atoms[atom] |= values
         return self
 
     def get_wcnfplus(self) -> WCNFPlus:
         formula = WCNFPlus()
-        for clause in self.get_clauses():
+        for clause in self.clauses:
             if isinstance(clause, Formula.MetaClause) and clause.weight is not None:
                 formula.append(clause.intclause, weight=clause.weight)
             elif isinstance(clause, Formula.MetaClause) and clause.atmost is not None:
@@ -279,7 +317,7 @@ class Formula:
 
     def append(
         self,
-        clause: Iterable[str],
+        clause: Iterable[Token],
         from_atom: Optional[str] = None,
         desc: Optional[str] = None,
         weight=None,
@@ -288,46 +326,42 @@ class Formula:
         self.clauses.append(Formula.MetaClause(from_atom, desc, clause, weight, atmost))
         self.__update_for_clause__(clause)
 
-    def append_dep(self, clause: Iterable[str], from_atom: str, dependency: Atom):
+    def append_dep(self, clause: Iterable[Token], from_atom: str, dependency: Atom):
         self.clauses.append(Formula.DepClause(from_atom, clause, dependency))
         self.__update_for_clause__(clause)
 
-    def append_blocker(self, clause: Iterable[str], from_atom: str, blocked: Atom):
+    def append_blocker(self, clause: Iterable[Token], from_atom: str, blocked: Atom):
         self.clauses.append(Formula.BlockerClause(from_atom, clause, blocked))
         self.__update_for_clause__(clause)
 
     def append_required_use(
         self,
-        clause: Iterable[str],
+        clause: Iterable[Token],
         from_atom: Atom,
-        flag: str,
+        flag: Token,
         tokens: List[Union[str, List]],
     ):
         self.clauses.append(Formula.RequiredUseClause(from_atom, clause, flag, tokens))
         self.__update_for_clause__(clause)
 
     def append_usedep(
-        self, clause: Iterable[str], from_atom: str, dep_atom: Atom, flag: str
+        self, clause: Iterable[Token], from_atom: str, dep_atom: Atom, flag: FlagToken
     ):
         self.clauses.append(Formula.UseDepClause(from_atom, clause, dep_atom, flag))
         self.__update_for_clause__(clause)
 
-    def __update_for_clause__(self, clause: Iterable[str]):
+    def __update_for_clause__(self, clause: Iterable[Token]):
         for token in clause:
-            if not token.startswith("_") and not token.startswith("-_"):
-                if "[" not in token:
-                    atom = FQAtom(token.lstrip("-"))
-                    if atom.CPN in self.atoms:
-                        self.atoms[QualifiedAtom(atom.CPN)].add(atom)
-                    else:
-                        self.atoms[QualifiedAtom(atom.CPN)] = {atom}
-                else:
-                    self.flags.add(FQAtom(token.lstrip("-")))
+            if not isinstance(token, VariableToken):
+                if isinstance(token, FlagToken):
+                    self.flags.add(token.value)
+                elif isinstance(token, AtomToken):
+                    self.atoms[QualifiedAtom(token.value.CPN)].add(token.value)
 
     def extend(
         self,
         from_atom: Atom,
-        clauses: List[List[str]],
+        clauses: List[List[Token]],
         desc: Optional[str] = None,
         weight=None,
         atmost=None,
@@ -335,21 +369,21 @@ class Formula:
         for clause in clauses:
             self.append(clause, from_atom, desc, weight, atmost)
 
-    def add_constraints(self, constraints: List[str]):
+    def add_constraints(self, constraints: List[Token]):
         self.__update_for_clause__(constraints)
         for clause in self.clauses:
             if not (
                 isinstance(clause, Formula.MetaClause) and clause.atmost is not None
             ):
                 clause.clause = list(clause.clause) + constraints
-                clause.requirements |= {negate(token) for token in constraints}
+                clause.requirements |= {token.neg() for token in constraints}
 
-    def get_clauses(self):
+    def make_numeric(self):
         for clause in self.clauses:
-            yield clause.str2num()
+            clause.str2num()
 
 
-def get_atmost_one_formulae(tokens: Sequence[str]) -> List[List[str]]:
+def get_atmost_one_formulae(tokens: Sequence[Token]) -> List[List[Token]]:
     """
     Returns a list of clauses that enforce that at most one of the tokens may be true
 
@@ -364,32 +398,19 @@ def get_atmost_one_formulae(tokens: Sequence[str]) -> List[List[str]]:
     # Enforce that for any two tokens in the list, one must be false
     for token in tokens[1:]:
         # Invert value of firsttoken
-        if tokens[0].startswith("-"):
-            firsttoken = tokens[0].lstrip("-")
-        else:
-            firsttoken = "-" + tokens[0]
+        firsttoken = tokens[0].neg()
 
         # Invert value of the other token
-        if token.startswith("-"):
-            othertoken = token.lstrip("-")
-        else:
-            othertoken = "-" + token
+        othertoken = token.neg()
         result.append([firsttoken, othertoken])
 
     return result + get_atmost_one_formulae(tokens[1:])
 
 
-def toggle(token: str) -> str:
-    if token.startswith("-"):
-        return token[1:]
-    else:
-        return "-" + token
-
-
-def toggle_clause(tokens: List[str]) -> List[str]:
+def toggle_clause(tokens: List[Token]) -> List[Token]:
     newtokens = []
     for token in tokens:
-        newtokens.append(toggle(token))
+        newtokens.append(token.neg())
     return newtokens
 
 
@@ -406,7 +427,7 @@ def get_required_use_formula(
 
     def get_required_use_formula_inner(
         tokens: List[Union[str, List]]
-    ) -> Tuple[Formula, List[str]]:
+    ) -> Tuple[Formula, List[Token]]:
         formula = Formula()
         clausevars = []
 
@@ -415,12 +436,8 @@ def get_required_use_formula(
                 if token[0] != "??" and token[0].endswith("?"):
                     newvar = Formula.genvariable([token])
                     subformulae, subvars = get_required_use_formula_inner(token[1:])
-                    if token[0].startswith("!"):
-                        usedep = mod.ATOM + "[" + token[0].lstrip("!").rstrip("?") + "]"
-                        subformulae.add_constraints([usedep, "-" + newvar])
-                    else:
-                        usedep = "-" + mod.ATOM + "[" + token[0].rstrip("?") + "]"
-                        subformulae.add_constraints([usedep, "-" + newvar])
+                    usedep = cond_flagstr(mod.ATOM, token[0].rstrip("?")).neg()
+                    subformulae.add_constraints([usedep, newvar.neg()])
                     # for clause in get_atmost_one_formulae(subvars):
                     #    formula.append_required_use(
                     #        ["-" + newvar] + clause, mod.ATOM, token
@@ -438,13 +455,13 @@ def get_required_use_formula(
                     if token[0] in ("??", "^^"):
                         for clause in get_atmost_one_formulae(subvars):
                             formula.append(
-                                ["-" + newvar] + clause,
+                                [newvar.neg()] + clause,
                                 mod.ATOM,
                                 human_readable_required_use(tokens),
                             )
                     if token[0] in ("||", "^^"):
                         formula.append(
-                            ["-" + newvar] + subvars,
+                            [newvar.neg()] + subvars,
                             mod.ATOM,
                             human_readable_required_use(tokens),
                         )
@@ -459,15 +476,12 @@ def get_required_use_formula(
                 flag = token.lstrip("!")
                 if use_expand:
                     flag = use_expand + "_" + flag
-                var = mod.ATOM + "[" + flag + "]"
+                var = fstr(mod.ATOM, flag)
                 if token.startswith("!"):
-                    formula.append_required_use(
-                        ["-" + var], mod.ATOM, "-" + var, tokens
-                    )
-                    clausevars.append("-" + var)
-                else:
-                    formula.append_required_use([var], mod.ATOM, var, tokens)
-                    clausevars.append(var)
+                    var = var.neg()
+
+                formula.append_required_use([var], mod.ATOM, var, tokens)
+                clausevars.append(var)
         return formula, clausevars
 
     formula, clausevars = get_required_use_formula_inner(tokens)
@@ -477,6 +491,29 @@ def get_required_use_formula(
     return formula
 
 
+def fstr(atom: FQAtom, flag: str) -> FlagToken:
+    """
+    Produces a flag token for the formula given an atom and a flag
+
+    This function does not produce disabled tokens. If a disabled token is
+    desired, the result should be negated.
+    """
+    assert flag[0] not in ("-", "!") and flag[-1] not in ("?", "=")
+    return FlagToken(atom.use(flag))
+
+
+def cond_flagstr(atom: FQAtom, flag: str, negtoken: str = "!") -> FlagToken:
+    """
+    Given an atom and a flag from a use conditional,
+    produces a token for use in the dependency formula
+    """
+    disabled = flag.startswith(negtoken)
+    flag = flag.lstrip(negtoken)
+    if disabled:
+        return fstr(atom, flag).neg()
+    return fstr(atom, flag)
+
+
 def get_dep_formula(mod: Pybuild, tokens) -> Tuple[Formula, Set[FQAtom]]:
     """
     Adds clauses to the given formula for the dependency strings of the given mod
@@ -486,31 +523,6 @@ def get_dep_formula(mod: Pybuild, tokens) -> Tuple[Formula, Set[FQAtom]]:
             corresponding to brackets in the original string
     """
 
-    def fstr(atom: Atom, flag: str) -> str:
-        """
-        Produces a flag token for the formula given an atom and a flag
-
-        This function does not produce disabled tokens. If a disabled token is
-        desired, "-" should be applied to the result.
-        """
-        return atom + "[" + flag.rstrip("?=").lstrip("!-") + "]"
-
-    def cond_flagstr(atom: Atom, flag: str) -> str:
-        """
-        Given an atom and a flag from a use conditional,
-        produces a token for use in the dependency formula
-        """
-        disabled = flag.startswith("!")
-        flag = flag.rstrip("?").lstrip("!")
-        # Note: If flag was disabled, we want the flag enabled in the clause, as it
-        # should either be enabled,
-        # or some other part of the clause must be true if disabled
-        if disabled:
-            return fstr(atom, flag)
-        # Note: If flag was enabled, we produce the clause (flag => dep),
-        # which is equivalent to (-flag | dep)
-        return "-" + fstr(atom, flag)
-
     formula = Formula()
     deps: Set[FQAtom] = set()
 
@@ -529,7 +541,7 @@ def get_dep_formula(mod: Pybuild, tokens) -> Tuple[Formula, Set[FQAtom]]:
                     # Either one part of the or must be true, or the variable for
                     # the clause should be false
                     new_formula, new_deps = get_dep_formula(mod, [subclause])
-                    new_formula.add_constraints(["-" + var])
+                    new_formula.add_constraints([var.neg()])
                     formula.merge(new_formula)
                     deps |= new_deps
 
@@ -539,7 +551,15 @@ def get_dep_formula(mod: Pybuild, tokens) -> Tuple[Formula, Set[FQAtom]]:
                 formula.append(orvars, mod.ATOM, human_readable_required_use([token]))
             elif token[0].endswith("?"):
                 new_formula, new_deps = get_dep_formula(mod, token[1:])
-                new_formula.add_constraints([cond_flagstr(mod.ATOM, token[0])])
+
+                # Note: If flag was disabled, we want the flag enabled in the clause, as it
+                # should either be enabled,
+                # or some other part of the clause must be true if disabled
+                # If flag was enabled, we produce the clause (flag => dep),
+                # which is equivalent to (-flag | dep)
+                new_formula.add_constraints(
+                    [cond_flagstr(mod.ATOM, token[0].rstrip("?")).neg()]
+                )
                 formula.merge(new_formula)
                 deps |= new_deps
             else:
@@ -553,10 +573,11 @@ def get_dep_formula(mod: Pybuild, tokens) -> Tuple[Formula, Set[FQAtom]]:
 
             # Note that load_pkg will only return mods that completely match atom
             # I.e. it will handle any versioned atoms itself
-            specificpkgs = [m for m in load_pkg(atom)]
+            specificpkgs = load_pkg(atom)
             specificatoms = [m.ATOM for m in load_pkg(atom)]
 
-            deps |= set(specificatoms)
+            if not blocker:
+                deps |= set(specificatoms)
 
             # !!foo[A,B] is equivalent to
             # || ( !!foo foo[-A] foo[-B] )
@@ -567,10 +588,14 @@ def get_dep_formula(mod: Pybuild, tokens) -> Tuple[Formula, Set[FQAtom]]:
             if blocker and not atom.USE:
                 for specatom in specificatoms:
                     formula.append_blocker(
-                        ["-" + specatom], mod.ATOM, Atom(token.lstrip("!"))
+                        [AtomToken(specatom, polarity=False)],
+                        mod.ATOM,
+                        Atom(token.lstrip("!")),
                     )
             elif not blocker:
-                formula.append_dep(specificatoms, mod.ATOM, atom)
+                formula.append_dep(
+                    [AtomToken(atom) for atom in specificatoms], mod.ATOM, atom
+                )
 
             # For each use flag dependency, add a requirement that the flag must be set
             # This depends on the operators used on the flag. See PMS 8.2.6.4
@@ -585,45 +610,32 @@ def get_dep_formula(mod: Pybuild, tokens) -> Tuple[Formula, Set[FQAtom]]:
                     if flag.lstrip("-") not in spec.IUSE_EFFECTIVE:
                         continue
 
-                    if flag.startswith("-"):  # dep[-flag]
-                        # 2-style disabled
-                        if blocker:
-                            new_formula.append_usedep(
-                                [fstr(spec.ATOM, flag)],
-                                mod.ATOM,
-                                atom.strip_use(),
-                                flag,
-                            )
-                        else:
-                            new_formula.append_usedep(
-                                ["-" + fstr(spec.ATOM, flag)],
-                                mod.ATOM,
-                                atom.strip_use(),
-                                flag,
-                            )
-                    else:  # dep[flag]
-                        # 2-style enabled
-                        if blocker:
-                            new_formula.append_usedep(
-                                ["-" + fstr(spec.ATOM, flag)],
-                                mod.ATOM,
-                                spec.ATOM.strip_use(),
-                                flag,
-                            )
-                        else:
-                            new_formula.append_usedep(
-                                [fstr(spec.ATOM, flag)],
-                                mod.ATOM,
-                                spec.ATOM.strip_use(),
-                                flag,
-                            )
+                    # 2-style
+                    if blocker:
+                        flagdep = cond_flagstr(spec.ATOM, flag, negtoken="-").neg()
+                        new_formula.append_usedep(
+                            [flagdep],
+                            mod.ATOM,
+                            atom.strip_use(),
+                            flagdep,
+                        )
+                    else:
+                        flagdep = cond_flagstr(spec.ATOM, flag, negtoken="-")
+                        new_formula.append_usedep(
+                            [flagdep],
+                            mod.ATOM,
+                            atom.strip_use(),
+                            flagdep,
+                        )
 
-                    new_formula.add_constraints(["-" + spec.ATOM])
+                    new_formula.add_constraints([AtomToken(spec.ATOM, polarity=False)])
                     formula.merge(new_formula)
     return formula, deps
 
 
-def generate_formula(mods: Iterable[Target], depsadded: Set[FQAtom]) -> Formula:
+def generate_formula(
+    mods: Iterable[Target], depsadded: Set[FQAtom], deep: bool
+) -> Formula:
     """
     Generates a hard dependency formula for the given mods
 
@@ -644,21 +656,20 @@ def generate_formula(mods: Iterable[Target], depsadded: Set[FQAtom]) -> Formula:
             # installed
             # Otherwise, we only include it for the purpose of dependency resolution
             formula.append_dep(
-                [mod.ATOM for mod in target.pkgs], target.source, target.atom
+                [AtomToken(mod.ATOM) for mod in target.pkgs], target.source, target.atom
             )
             if target.atom.USE:
                 for flag in target.atom.USE:
-                    sflag = flag.lstrip("-")
-                    if flag.startswith("-"):
-                        prefix = "-"
-                    else:
-                        prefix = ""
                     for mod in target.pkgs:
+                        flagdep = cond_flagstr(mod.ATOM, flag, negtoken="-")
                         formula.append_usedep(
-                            ["-" + mod.ATOM, prefix + mod.ATOM + f"[{sflag}]"],
+                            [
+                                AtomToken(mod.ATOM, polarity=False),
+                                flagdep,
+                            ],
                             target.source,
                             target.atom.strip_use(),
-                            flag,
+                            flagdep,
                         )
 
         new += target.pkgs
@@ -683,26 +694,23 @@ def generate_formula(mods: Iterable[Target], depsadded: Set[FQAtom]) -> Formula:
                         use_expand="texture_size",
                     )
                 )
-            new_formula.add_constraints([f"-{mod.ATOM}"])
+            new_formula.add_constraints([AtomToken(mod.ATOM, polarity=False)])
             formula.merge(new_formula)
             for flag in get_forced_use(mod.ATOM):
                 if flag.lstrip("-") not in mod.IUSE_EFFECTIVE:
                     continue
 
-                if flag[0] == "-":
-                    formula.append(
-                        [f"-{mod.ATOM}[{flag[1:]}]"],
-                        "profile use.force or mod.use.force",
-                        f"Flag {flag} is forced on mod {mod.ATOM}",
-                    )
-                else:
-                    formula.append(
-                        [f"{mod.ATOM}[{flag}]"], "use.force", "Forced flag from profile"
-                    )
+                formula.append(
+                    [cond_flagstr(mod.ATOM, flag, negtoken="-")],
+                    "profile use.force or mod.use.force",
+                    f"Flag {flag} is forced on mod {mod.ATOM}",
+                )
 
             depsadded.add(mod.ATOM)
-            # Add this mod's dependencies to the next set of mods to parse
-            nextmods |= deps
+            if deep:
+                # Add this mod's dependencies to the next set of mods to parse
+                nextmods |= deps
+
         new = []
         for atom in nextmods:
             if atom not in depsadded:
@@ -741,7 +749,9 @@ def add_masked(atoms: Dict[QualifiedAtom, Set[FQAtom]], flags: Iterable[FQAtom])
                             )
                         else:
                             formula.append_blocker(
-                                ["-" + atom], description, masked_atom
+                                [AtomToken(atom, polarity=False)],
+                                description,
+                                masked_atom,
                             )
 
     return formula
