@@ -13,7 +13,8 @@ from portmod.repo.metadata import get_use_flag_atom_aliases
 from portmod.util import get_keyword_dep
 from portmodlib.atom import Atom, FQAtom, QualifiedAtom, version_gt
 
-from .formula import Formula, toggle, toggle_clause
+from .formula import Formula, cond_flagstr, toggle_clause
+from .tokens import AtomToken, FlagToken, Token
 
 WEIGHTS = SimpleNamespace()
 
@@ -39,7 +40,6 @@ def weigh_clauses(
     deep: bool,
     depclean: bool,
     update: bool,
-    newuse: bool,
     emptytree: bool,
 ) -> Formula:
     """Creates soft clauses for the given atoms"""
@@ -59,13 +59,20 @@ def weigh_clauses(
                 and not is_selected(atom)
                 and Atom(atom.CPN) not in explicit
             ):
-                formula.append(["-" + atom], weight=WEIGHTS.not_needed)
+                formula.append(
+                    [AtomToken(atom, polarity=False)], weight=WEIGHTS.not_needed
+                )
                 total_not_needed += WEIGHTS.not_needed
 
     # When deep depcleaning, keep flags as they are,
     # so the only change should be packages being removed.
     flag_formula, total_flag_weight = weigh_flags(
-        flags, atoms, explicit, newuse=newuse, freeze=depclean and deep and not newuse
+        flags,
+        atoms,
+        explicit,
+        update=update,
+        deep=deep,
+        freeze=depclean and deep and not update,
     )
     formula.merge(flag_formula)
 
@@ -105,7 +112,7 @@ def weigh_clauses(
             elif not version_gt(vy, vx):
                 # If nether is greater than the other, they are equal
                 # The installed version comes first, unless the package was selected
-                if atom.CPN in explicit or emptytree:
+                if (atom.CPN in explicit and not update) or emptytree:
                     if rx is None:
                         return 1
                     if ry is None:
@@ -131,7 +138,7 @@ def weigh_clauses(
 
             inorder = sorted(tosort, key=cmp_to_key(cmp))
             for index, (version, repo, atom) in enumerate(inorder):
-                formula.append(["-" + atom], weight=i)
+                formula.append([AtomToken(atom, polarity=False)], weight=i)
                 if (
                     index < len(inorder) - 1
                     and inorder[index + 1][0] == version
@@ -150,7 +157,7 @@ def weigh_clauses(
 
         # At most one version of the mod must be installed
         formula.append(
-            atoms[group],
+            [AtomToken(atom) for atom in atoms[group]],
             "inviolable-rule",
             "At most one version may be installed",
             atmost=1,
@@ -166,18 +173,20 @@ def weigh_clauses(
             and Atom(mod.CPN) not in explicit
             and not emptytree
         ):
-            if update:  # If update is passed, penalize removal, but updates override
-                formula.append([mod.ATOM], weight=WEIGHTS.keep_installed)
+            # If update and deep are passed, penalize removal, but updates override
+            if update and deep:
+                formula.append([AtomToken(mod.ATOM)], weight=WEIGHTS.keep_installed)
             elif depclean and deep:
                 for pkg in load_pkg(Atom(mod.CPN)):
                     formula.append(
-                        ["-" + pkg.ATOM],
+                        [AtomToken(pkg.ATOM, polarity=False)],
                         weight=WEIGHTS.keep_installed + total_up_weight,
                     )
                     total_depclean_weight += total_up_weight
             else:  # if update is not passed, stay installed regardless of updates
                 formula.append(
-                    [mod.ATOM], weight=WEIGHTS.keep_installed + total_up_weight
+                    [AtomToken(mod.ATOM)],
+                    weight=WEIGHTS.keep_installed + total_up_weight,
                 )
     # Packages which are already installed should, if not explicitly depcleaned,
     # be kep in their current state. This must override the weight of packages to be removed,
@@ -186,11 +195,12 @@ def weigh_clauses(
         if (
             is_selected(pkg.ATOM)
             and Atom(pkg.CPN) not in explicit
-            and not update
+            and not (update and deep)
             and not emptytree
         ):
             formula.append(
-                [pkg.ATOM], weight=WEIGHTS.keep_installed + total_depclean_weight
+                [AtomToken(pkg.ATOM)],
+                weight=WEIGHTS.keep_installed + total_depclean_weight,
             )
 
     # When depcleaning, make sure the weight of installing not installed mods is higher
@@ -205,7 +215,7 @@ def weigh_clauses(
                     and Atom(atom.CPN) not in explicit
                 ):
                     formula.append(
-                        ["-" + atom],
+                        [AtomToken(atom, polarity=False)],
                         weight=WEIGHTS.not_needed
                         + total_up_weight
                         + WEIGHTS.keep_installed,
@@ -218,18 +228,21 @@ def weigh_flags(
     atoms: Mapping[QualifiedAtom, Iterable[FQAtom]],
     explicit: AbstractSet[Atom],
     *,
-    newuse: bool,
+    update: bool,
     freeze: bool,
+    deep: bool,
 ):
     formula = Formula()
 
     def iterate_flags():
         for flagatom in flags:
             flag = list(flagatom.USE)[0]  # Note: each only has one flag
-            enabledatom = flagatom.lstrip("-")
+            assert not flag.startswith("-")
+            assert not flagatom.startswith("-")
+            enabledatom = FlagToken(flagatom)
             atom_nouse = flagatom.strip_use()
             pkg = load_pkg_fq(atom_nouse)
-            yield flag, atom_nouse, enabledatom, pkg
+            yield flag, AtomToken(atom_nouse), enabledatom, pkg
 
         # We also need to consider flags which don't show up in DEPEND-style strings
         # since flags in IUSE may have an alias
@@ -239,7 +252,7 @@ def weigh_flags(
                 for flag in pkg.IUSE_EFFECTIVE:
                     flagatom = FQAtom(f"{pkg.ATOM}[{flag}]")
                     if flagatom not in flags:
-                        yield flag, pkg.ATOM, flagatom, pkg
+                        yield flag, AtomToken(pkg.ATOM), FlagToken(flagatom), pkg
 
     default_weight = WEIGHTS.default_flag
     total_flag_weight = 0
@@ -256,11 +269,15 @@ def weigh_flags(
             enabled = pkg.get_use()
 
             if flag in enabled:
-                formula.append(["-" + atom, flagatom])
+                formula.append([atom.neg(), flagatom])
             else:
-                formula.append(["-" + atom, "-" + flagatom])
+                formula.append([atom.neg(), flagatom.neg()])
 
-        elif newuse or atom.CPN in explicit or not load_installed_pkg(pkg.CPN):
+        elif (
+            (deep and update)
+            or atom.value.CPN in explicit
+            or not load_installed_pkg(pkg.CPN)
+        ):
             if pkg.REPO not in repo_aliases:
                 repo_aliases[pkg.REPO] = get_use_flag_atom_aliases(
                     get_repo(pkg.REPO).location
@@ -276,13 +293,13 @@ def weigh_flags(
                 pass
             elif "-" + flag in global_user_flags:
                 pass
-            elif flag in aliases and aliases[flag].CPN != atom.CPN:
+            elif flag in aliases and aliases[flag].CPN != atom.value.CPN:
                 alias = aliases[flag]
 
                 if alias.CPN not in atoms:
                     # Package isn't installed, or even in the dependency tree,
                     # so default to disabled
-                    formula.append(["-" + flagatom], weight=default_weight)
+                    formula.append([flagatom.neg()], weight=default_weight)
                     continue
 
                 desc = [
@@ -290,9 +307,9 @@ def weigh_flags(
                     "Should never cause a conflict",
                 ]
 
-                aliasdeps: List[List[str]] = []
+                aliasdeps: List[List[Token]] = []
                 for aliased_pkg in load_pkg(alias):
-                    tokens: List[str] = []
+                    tokens: List[Token] = []
                     if alias.USE:
                         # List of tokens relevant to the use flag, plus a variable which will be
                         # used later to reference it
@@ -301,21 +318,17 @@ def weigh_flags(
                         # If all the parts are present, it's true
                         tokens.append(formula.genvariable(desc))
                         for alias_flag in alias.USE:
-                            if alias_flag.startswith("-"):
-                                alias_flag.strip("-")
-                                tokens.append(
-                                    "-" + aliased_pkg.ATOM + "[" + alias_flag + "]"
-                                )
-                            else:
-                                tokens.append(aliased_pkg.ATOM + "[" + alias_flag + "]")
+                            tokens.append(
+                                cond_flagstr(aliased_pkg.ATOM, alias_flag, negtoken="-")
+                            )
 
-                    tokens.append(aliased_pkg.ATOM)
+                    tokens.append(AtomToken(aliased_pkg.ATOM))
                     aliasdeps.append(tokens)
 
                 # If no package matching the alias is installed,
                 # this will give weight to the flag being disabled.
                 formula.append(
-                    ["-" + flagatom] + [clause[0] for clause in aliasdeps],
+                    [flagatom.neg()] + [clause[0] for clause in aliasdeps],
                     weight=default_weight,
                 )
                 for clause in aliasdeps:
@@ -325,7 +338,7 @@ def weigh_flags(
 
                         # This makes the variable at the beginning false if any of the parts are false
                         for term in clause[1:]:
-                            formula.append(["-" + clause[0], term])
+                            formula.append([clause[0].neg(), term])
 
                 # var is true iff no packages matching the alias are installed
                 var = formula.genvariable(desc)
@@ -340,13 +353,13 @@ def weigh_flags(
 
                 for clause in aliasdeps:
                     # If package is installed, var must be false
-                    formula.append(["-" + var, toggle(clause[0])])
+                    formula.append([var.neg(), clause[0].neg()])
 
             elif flag in get_use(pkg)[0]:  # default value
-                formula.append(["-" + atom, flagatom], weight=default_weight)
+                formula.append([atom.neg(), flagatom], weight=default_weight)
                 total_flag_weight += default_weight
             else:
-                formula.append(["-" + atom, "-" + flagatom], weight=default_weight)
+                formula.append([atom.neg(), flagatom.neg()], weight=default_weight)
                 total_flag_weight += default_weight
         else:
             # If not running in deep mode, keep flags the same for
@@ -354,27 +367,31 @@ def weigh_flags(
             enabled = pkg.get_use()
 
             if flag in enabled:
-                formula.append(["-" + atom, flagatom], weight=default_weight)
+                formula.append([atom.neg(), flagatom], weight=default_weight)
             else:
-                formula.append(["-" + atom, "-" + flagatom], weight=default_weight)
+                formula.append([atom.neg(), flagatom.neg()], weight=default_weight)
 
     user_weight = WEIGHTS.user_flag + total_flag_weight
     # Clauses for user flags. These will always override changes from the default value.
     for flag, atom, flagatom, mod in iterate_flags():
-        if newuse or atom.CPN in explicit or not load_installed_pkg(mod.CPN):
+        if (
+            (deep and update)
+            or atom.value.CPN in explicit
+            or not load_installed_pkg(mod.CPN)
+        ):
             user_flags = get_user_use(mod.ATOM)
             global_user_flags = get_user_global_use()
             if flag in user_flags:
-                formula.append(["-" + atom, flagatom], weight=user_weight)
+                formula.append([atom.neg(), flagatom], weight=user_weight)
                 total_user_weight += user_weight
             elif "-" + flag in user_flags:
-                formula.append(["-" + atom, "-" + flagatom], weight=user_weight)
+                formula.append([atom.neg(), flagatom.neg()], weight=user_weight)
                 total_user_weight += user_weight
             elif flag in global_user_flags:
-                formula.append(["-" + atom, flagatom], weight=user_weight)
+                formula.append([atom.neg(), flagatom], weight=user_weight)
                 total_user_weight += user_weight
             elif "-" + flag in global_user_flags:
-                formula.append(["-" + atom, "-" + flagatom], weight=user_weight)
+                formula.append([atom.neg(), flagatom.neg()], weight=user_weight)
                 total_user_weight += user_weight
 
     return formula, total_user_weight
