@@ -16,7 +16,7 @@ from portmodlib.util import pybuild_dumper
 from .config import get_config
 from .globals import env, get_version
 from .pybuild import InstalledPybuild, Pybuild, to_cache
-from .repo.loader import __safe_load_file, get_atom_from_path
+from .repo.loader import _safe_load_file, get_atom_from_path
 
 
 class PreviouslyEncounteredException(Exception):
@@ -61,7 +61,7 @@ def clear_cache_for_path(path: str):
     cache.clear_path(path)
 
 
-def cache_valid(path: str) -> bool:
+def cache_valid(pybuild_path: str, cache_path: str) -> bool:
     """
     Determines if the cache file at the given path is valid
 
@@ -69,10 +69,10 @@ def cache_valid(path: str) -> bool:
     the version of portmod is the same as the version stored in the cache file
     and that all file hashes in the cache file are valid
     """
-    if not os.path.exists(path):
+    if not os.path.exists(cache_path):
         return False
 
-    with open(path, "r", encoding="utf-8") as cache_file:
+    with open(cache_path, "r", encoding="utf-8") as cache_file:
         try:
             mod = json.load(cache_file)
         except Exception:
@@ -81,6 +81,9 @@ def cache_valid(path: str) -> bool:
     if mod.get("__portmod_version__") != get_version():
         return False
 
+    if not os.path.samefile(mod.get("FILE"), pybuild_path):
+        return False
+
     if not mod.get("__hashes__", []):
         return False
 
@@ -135,7 +138,7 @@ def load_cache(path: str, installed: bool) -> Pybuild:
     else:
         cache_file = os.path.join(env.PYBUILD_CACHE_DIR, repo_name, atom.C, atom.PF)
 
-    if not cache_valid(cache_file):
+    if not cache_valid(path, cache_file):
         from .loader import SandboxedError
 
         os.makedirs(os.path.dirname(cache_file), exist_ok=True)
@@ -143,7 +146,7 @@ def load_cache(path: str, installed: bool) -> Pybuild:
         if not path.endswith(".pybuild"):
             raise Exception()
         try:
-            cache_str = create_cache_str(__safe_load_file(path, installed=installed))
+            cache_str = create_cache_str(_safe_load_file(path, installed=installed))
         except SandboxedError as e:
             cache._failed[path] = e
             raise e
