@@ -5,13 +5,16 @@
 Helper functions for interacting with package licenses
 """
 
-from functools import lru_cache
-from typing import List, Union
+import os
+from typing import AbstractSet, List, Union
 
 from portmod.config import get_config
 from portmod.config.use import get_use
+from portmod.functools import system_cache
+from portmod.globals import env
+from portmod.parsers.flags import get_flags
 from portmod.pybuild import Pybuild
-from portmod.repo import get_repo, get_repo_root
+from portmod.repo import get_repo
 from portmod.repo.metadata import get_license_groups
 from portmodlib.usestr import use_reduce
 
@@ -20,20 +23,30 @@ def _get_pkg_license_groups(pkg: Pybuild):
     if pkg.INSTALLED:
         return get_license_groups(get_repo(pkg.REPO).location)
     else:
-        return get_license_groups(get_repo_root(pkg.FILE))
+        if not pkg.REPO_PATH:
+            raise RuntimeError(
+                "Checking license of a package with no repository! "
+                "This should not happen!"
+            )
+        return get_license_groups(pkg.REPO_PATH)
 
 
-def is_license_accepted(mod: Pybuild) -> bool:
+def _user_package_accept_license_path() -> str:
+    return os.path.join(env.prefix().CONFIG_DIR, "package.accept_license")
+
+
+def is_license_accepted(pkg: Pybuild, enabled_flags: AbstractSet[str]) -> bool:
     """
-    Returns true if the mod's license(s) are accepted by the user's configuration
+    Returns true if the package's license(s) are accepted by the user's configuration
 
     For a license to be accepted, it must be both listed, either explicitly,
     part of a group, or with the * wildcard  and it must not be blacklisted
     by a license or license group prefixed by a '-'
     """
-    license_groups = _get_pkg_license_groups(mod)
+    license_groups = _get_pkg_license_groups(pkg)
 
-    ACCEPT_LICENSE = get_config()["ACCEPT_LICENSE"]
+    ACCEPT_LICENSE = get_config()["ACCEPT_LICENSE"].split()
+    package_accept_license = get_flags(_user_package_accept_license_path(), pkg.ATOM)
 
     def accepted(group: Union[str, List]) -> bool:
         if not group:
@@ -42,13 +55,13 @@ def is_license_accepted(mod: Pybuild) -> bool:
         if isinstance(group, str):
             allowed = False
             # Check if license is allowed by anything in ACCEPT_LICENSE
-            for license in ACCEPT_LICENSE:
+            for license in list(ACCEPT_LICENSE) + list(package_accept_license):
                 if license.startswith("-") and (
                     license == group
                     or (license[1] == "@" and group in license_groups[license[2:]])
                 ):
-                    # not allowed if matched by this
-                    return False
+                    # not allowed if matched by this, but can be overridden later
+                    allowed = False
                 if license == "*":
                     allowed = True
                 if license.startswith("@") and group in license_groups[license[1:]]:
@@ -59,13 +72,10 @@ def is_license_accepted(mod: Pybuild) -> bool:
 
         return all(accepted(license) for license in group)
 
-    enabled, disabled = get_use(mod)
-    return accepted(use_reduce(mod.LICENSE, enabled, disabled, opconvert=True))
-
-    # TODO: implement package-specific license acceptance via package.license config file
+    return accepted(use_reduce(pkg.LICENSE, enabled_flags, opconvert=True))
 
 
-@lru_cache()
+@system_cache
 def has_eula(package: Pybuild) -> bool:
     groups = _get_pkg_license_groups(package)
 
