@@ -27,7 +27,7 @@ from typing import (
 )
 
 import portmod
-from portmod.config import get_config
+from portmod.config import get_config, variable_data_dir
 from portmod.execute import sandbox_execute
 from portmod.functools import install_cache, prefix_aware_cache, system_cache
 from portmod.vdb import vdb_path
@@ -127,7 +127,16 @@ def load_module(file: str, state):
     from portmod.modules import Module, ModuleFunction
 
     perms = Permissions(
-        rw_paths=[state.CACHE, state.ROOT], global_read=True, tmp=state.TEMP
+        rw_paths=[state.CACHE, state.ROOT],
+        ro_paths=[
+            # Builtin sets and the package database can't be modified by modules
+            os.path.join(variable_data_dir(), "sets", "world"),
+            os.path.join(variable_data_dir(), "sets", "world_sets"),
+            os.path.join(variable_data_dir(), "db"),
+            os.path.join(variable_data_dir(), "news"),
+        ],
+        global_read=True,
+        tmp=state.TEMP,
     )
 
     module_data = _sandbox_execute_module(file, "load", permissions=perms)
@@ -513,6 +522,7 @@ def load_all(
         yield mod
 
 
+# TODO: Deprecated. Remove in 3.0
 @prefix_aware_cache
 def _load_local_pkg(package_path: str) -> InstalledPybuild:
     name = os.path.basename(package_path)
@@ -542,6 +552,9 @@ def _load_local_pkg(package_path: str) -> InstalledPybuild:
         INSTALL_DIRS=[install_dir],
         FILE=package_path,
         PROPERTIES="local",
+        _PYBUILD_VER=1,
+        TIER="a",
+        DATA_OVERRIDES="",
         REPO="",
     )
 
@@ -555,6 +568,7 @@ def load_all_installed() -> Generator[InstalledPybuild, None, None]:
         if mod:
             yield mod
 
+    # TODO: Deprecated. Remove in 3.0
     local_dir = os.path.join(env.prefix().LOCAL_MODS)
     if os.path.exists(local_dir):
         for subdir in os.listdir(local_dir):
