@@ -174,7 +174,7 @@ class exclusive_lock(ContextDecorator):
         while not self.lock.acquire():
             if not bar:
                 bar = _get_bar().start()
-            bar.update(status=l10n("acquiring-exclusive"))
+            bar.update(status=l10n("acquiring-exclusive", prefix=self.prefix))
         if bar:
             bar.finish()
         return self
