@@ -15,6 +15,13 @@ from portmodlib.colour import bright, green, lblue, lgreen, red, yellow
 from portmodlib.l10n import l10n
 from portmodlib.module_util import _add_redirection
 
+from ._deprecated.rebuild import get_rebuild_manifest
+from ._deprecated.vfs import (
+    clear_vfs_sort,
+    require_vfs_sort,
+    sort_vfs,
+    vfs_needs_sorting,
+)
 from ._deps import resolve
 from .cfg_protect import get_protected_path, get_redirections, handle_cfg_protect
 from .config import get_config
@@ -38,7 +45,6 @@ from .parsers.flags import add_flag
 from .perms import Permissions
 from .prompt import prompt_bool
 from .query import FlagDesc, LocalFlagDesc, get_flag_desc
-from .rebuild import get_rebuild_manifest
 from .repo import get_repo
 from .repo.keywords import _user_package_accept_keywords_path, add_keyword
 from .repo.loader import commit_moved
@@ -55,7 +61,6 @@ from .transactions import (
 from .tsort import CycleException
 from .util import KeywordDep, LicenseDep, select_package
 from .vdb import VDB
-from .vfs import clear_vfs_sort, require_vfs_sort, sort_vfs, vfs_needs_sorting
 
 
 def global_prepare():
@@ -144,7 +149,7 @@ def _update_configuration(
                         arch=env.prefix().ARCH,
                     )
                 )
-                return
+                sys.exit(1)
 
         print(l10n("necessary-keyword-changes"))
         for keyword in keyword_changes:
@@ -459,7 +464,7 @@ def configure(
                     state = {
                         "UNFETCHED": to_fetch,
                         "A": sources,
-                        "USE": trans.pkg.get_use(),
+                        "USE": trans.flags,
                     }
                     _sandbox_execute_pybuild(
                         trans.pkg.FILE,
@@ -607,7 +612,7 @@ def configure(
 
     # Check if packages were just modified and can be removed from the rebuild set
     # Any transaction type warrants removal, as they were either rebuilt,
-    # and thus can be removed, or deleted, and no longer need to be rebuild
+    # and thus can be removed, or deleted, and no longer need to be rebuilt
     for atom in get_set("rebuild"):
         installed_pkg = load_installed_pkg(atom)
         if not installed_pkg or installed_pkg.CPN in [
@@ -615,7 +620,12 @@ def configure(
         ]:
             remove_set("rebuild", atom)
 
-    info(l10n("checking-rebuild"))
+    for pkg in load_all_installed():
+        # If no pybuild1 packages are installed, the rebuild feature isn't needed
+        if pkg._PYBUILD_VER == 1:
+            info(l10n("checking-rebuild"))
+            break
+
     # Check if packages need to be added to rebuild set
     for pkg in load_all_installed():
         if pkg.CPN not in get_set("rebuild") and pkg.INSTALLED_REBUILD_FILES:
