@@ -19,15 +19,19 @@ from portmod.lock import exclusive_lock
 from portmodlib.atom import Atom
 from portmodlib.colour import blue, bright, green, yellow
 from portmodlib.l10n import get_locales, l10n
-from portmodlib.parsers.list import read_list
+from portmodlib.parsers.list import read_list, write_list
 from portmodlib.portmod import parse_news
 
-from .config import get_config
+from .config import get_config_value, variable_data_dir
+from .config.profiles import get_profile_path, profile_exists
 from .globals import env
 from .loader import load_installed_pkg
 from .prefix import get_prefixes
 from .repo import Repo
-from .repo.profiles import get_profile_path, profile_exists
+
+
+def _get_news_dir():
+    return os.path.join(variable_data_dir(), "news")
 
 
 def get_article_path(repo_root: str, article: str) -> str:
@@ -82,7 +86,7 @@ def is_news_visible(repo: Repo, article_name: str) -> bool:
     ):
         return False
 
-    ACCEPT_KEYWORDS = set(get_config()["ACCEPT_KEYWORDS"])
+    ACCEPT_KEYWORDS = set(get_config_value("ACCEPT_KEYWORDS"))
     if keyword_conditions and not any(
         keyword in ACCEPT_KEYWORDS or keyword.lstrip("~") in ACCEPT_KEYWORDS
         for keyword in keyword_conditions
@@ -96,7 +100,7 @@ def is_news_unread(repo: Repo, article: str) -> bool:
     """
     Returns true if the given news article needs to be read.
     """
-    unread_file = os.path.join(env.prefix().NEWS_DIR, "news-" + repo.name + ".unread")
+    unread_file = os.path.join(_get_news_dir(), "news-" + repo.name + ".unread")
     if not os.path.exists(unread_file) or article in read_list(unread_file):
         return True
 
@@ -105,24 +109,20 @@ def is_news_unread(repo: Repo, article: str) -> bool:
 
 def mark(repo: Repo, article: str, read=True):
     """Marks the given news article as read"""
-    os.makedirs(env.prefix().NEWS_DIR, exist_ok=True)
+    os.makedirs(_get_news_dir(), exist_ok=True)
     if read:
-        remove_file = os.path.join(
-            env.prefix().NEWS_DIR, "news-" + repo.name + ".unread"
-        )
-        add_file = os.path.join(env.prefix().NEWS_DIR, "news-" + repo.name + ".read")
+        remove_file = os.path.join(_get_news_dir(), "news-" + repo.name + ".unread")
+        add_file = os.path.join(_get_news_dir(), "news-" + repo.name + ".read")
     else:
-        add_file = os.path.join(env.prefix().NEWS_DIR, "news-" + repo.name + ".unread")
-        remove_file = os.path.join(env.prefix().NEWS_DIR, "news-" + repo.name + ".read")
+        add_file = os.path.join(_get_news_dir(), "news-" + repo.name + ".unread")
+        remove_file = os.path.join(_get_news_dir(), "news-" + repo.name + ".read")
 
     if os.path.exists(remove_file):
         contents = read_list(remove_file)
 
         if article in contents:
             del contents[contents.index(article)]
-            with open(remove_file, "w") as file:
-                for line in contents:
-                    print(line + "\n", file=file)
+            write_list(remove_file, contents)
         else:
             return
 
@@ -132,7 +132,7 @@ def mark(repo: Repo, article: str, read=True):
 
 def iterate_news(repo: Repo, unread_only=False, visible_only=False):
     news_dir = os.path.join(repo.location, "metadata", "news")
-    unread_file = os.path.join(env.prefix().NEWS_DIR, "news-" + repo.name + ".unread")
+    unread_file = os.path.join(_get_news_dir(), "news-" + repo.name + ".unread")
     if os.path.exists(unread_file):
         unread = set(read_list(unread_file))
     else:
@@ -149,9 +149,7 @@ def iterate_news(repo: Repo, unread_only=False, visible_only=False):
 def display_unread_message():
     total_unread = 0
     for repo in env.prefix().REPOS:
-        unread_file = os.path.join(
-            env.prefix().NEWS_DIR, "news-" + repo.name + ".unread"
-        )
+        unread_file = os.path.join(_get_news_dir(), "news-" + repo.name + ".unread")
         if os.path.exists(unread_file):
             unread = len(read_list(unread_file))
             if unread:
@@ -181,20 +179,20 @@ def update_news():
         env.set_prefix(prefix)
         with exclusive_lock():
             for repo in env.prefix().REPOS:
-                os.makedirs(env.prefix().NEWS_DIR, exist_ok=True)
+                os.makedirs(_get_news_dir(), exist_ok=True)
                 unread_file = os.path.join(
-                    env.prefix().NEWS_DIR, "news-" + repo.name + ".unread"
-                )
-                skip_file = os.path.join(
-                    env.prefix().NEWS_DIR, "news-" + repo.name + ".skip"
+                    _get_news_dir(), "news-" + repo.name + ".unread"
                 )
+                skip_file = os.path.join(_get_news_dir(), "news-" + repo.name + ".skip")
                 if os.path.exists(skip_file):
                     skip = set(read_list(skip_file))
                 else:
                     skip = set()
 
+                repo_news = set()
                 for news_dir in iterate_news(repo):
                     news_item = os.path.basename(news_dir)
+                    repo_news.add(news_item)
                     if news_item in skip:
                         continue
 
@@ -211,10 +209,16 @@ def update_news():
 
                 if not os.path.exists(unread_file):
                     # If unread file doesn't exist, this must be a newly added repository
-                    # In this case, all enws is old news, so it gets added to the skip file
+                    # In this case, all news is old news, so it gets added to the skip file
                     # above, but not to the unread file
                     with open(unread_file, "w") as file:
                         file.write("")
+                else:
+                    # Prune any stale entries from the unread file
+                    contents = read_list(unread_file)
+                    new_contents = [line for line in contents if line in repo_news]
+                    if contents != new_contents:
+                        write_list(unread_file, new_contents)
 
     env.set_prefix(orig_prefix)
     return True
