@@ -35,14 +35,14 @@ from portmodlib.fs import (
 from portmodlib.l10n import l10n
 from portmodlib.module_util import _add_redirection
 
+from ._deprecated.rebuild import get_rebuild_manifest
+from ._deprecated.vfs import _cleanup_tmp_archive_dir
 from .cache import clear_cache_for_path
 from .functools import clear_install_cache
 from .modules import module_prerm
 from .perms import Permissions
 from .prompt import prompt_options
 from .pybuild import InstalledPybuild, Pybuild
-from .rebuild import get_rebuild_manifest
-from .vfs import _cleanup_tmp_archive_dir
 
 
 def belongs(path: str, case_sensitive: bool = True) -> Optional[InstalledPybuild]:
@@ -130,8 +130,12 @@ def pkg_postinst(
 ):
     # Default does nothing
     if "pkg_postinst" in pkg.FUNCTIONS:
+        rw_paths = []
+        if pkg._PYBUILD_VER == 1:
+            rw_paths.append(final_install)
+
         permissions = Permissions(
-            rw_paths=[final_install],
+            rw_paths=rw_paths,
             global_read=True,
             network=False,
             tmp=state.T,
@@ -149,8 +153,11 @@ def pkg_postinst(
 def pkg_prerm(pkg: InstalledPybuild, root: str, state: PhaseState = PhaseState()):
     # Default does nothing
     if "pkg_prerm" in pkg.FUNCTIONS:
+        rw_paths = []
+        if pkg._PYBUILD_VER == 1:
+            rw_paths.append(root)
         permissions = Permissions(
-            rw_paths=[root], global_read=True, network=False, tmp=state.T
+            rw_paths=rw_paths, global_read=True, network=False, tmp=state.T
         )
         _sandbox_execute_pybuild(
             pkg.FILE,
@@ -425,6 +432,11 @@ def install_pkg(mod: Pybuild, use_flags: Set[str]):
         # unless source pybuild is in DB (i.e we're reinstalling)
         if not mod.FILE.startswith(db_path):
             shutil.copy(mod.FILE, db_path)
+            filespath = os.path.join(os.path.dirname(mod.FILE), "files")
+            if os.path.exists(filespath):
+                shutil.copytree(filespath, os.path.join(db_path, "files"))
+                vdb.git.add(os.path.join(mod.CATEGORY, mod.PN, "files"))
+
         vdb.git.add(os.path.join(mod.CATEGORY, mod.PN, os.path.basename(mod.FILE)))
 
         manifest_path = os.path.join(os.path.dirname(mod.FILE), "Manifest")
@@ -486,6 +498,7 @@ def install_pkg(mod: Pybuild, use_flags: Set[str]):
                 for entry in get_rebuild_manifest(environment["REBUILD_FILES"]):
                     manifest.add_entry(entry)
                 manifest.write(os.path.join(db_path, "REBUILD_FILES"))
+                vdb.git.add(os.path.join(db_path, "REBUILD_FILES"))
 
         clear_cache_for_path(os.path.join(db_path, os.path.basename(mod.FILE)))
 
