@@ -1,8 +1,13 @@
 # Copyright 2019-2021 Portmod Authors
 # Distributed under the terms of the GNU General Public License v3
 
+"""
+CLI prompt functions
+"""
+
 import re
 import sys
+from textwrap import fill
 from typing import AbstractSet, List, Optional, Sequence, Tuple
 
 from portmodlib.colour import bright, lblue, lgreen, lred
@@ -16,6 +21,15 @@ def display_num_list(
     *,
     start: int = 0,
 ):
+    """
+    Displays a numbered list to stdout
+
+    args:
+        items: The primary elements to be displayed
+        notes: A secondary note which, if provided, will be displayed in parentheses after the item
+        selected: Selected indices will be followed by an asterisk
+        start: The index to begin the list
+    """
     padding = len(str(len(items)))
 
     def print_item(index: int, item: str, note: Optional[str]):
@@ -72,7 +86,22 @@ def strtobool(value: str) -> bool:
     raise ValueError(f"Invalid boolean value {value}")
 
 
-def prompt_bool(question):
+def prompt_bool(question) -> bool:
+    """
+    Prompts the user for yes or no
+
+    See :func:`strtobool` for details of how the response is parsed
+
+    args:
+        question: The question to display prior to the prompt
+
+    raises:
+        EOFError: If the user enters -1 or EOF
+
+    returns:
+        True if the user answered yes, or false if the user answered no
+    """
+
     sys.stdout.write(
         "{} [{}/{}]: ".format(
             question, bright(lgreen(l10n("yes"))), bright(lred(l10n("no")))
@@ -80,7 +109,7 @@ def prompt_bool(question):
     )
     while True:
         try:
-            return strtobool(input().lower())
+            return strtobool(input().strip().lower())
         except ValueError:
             sys.stdout.write(
                 l10n(
@@ -92,41 +121,85 @@ def prompt_bool(question):
 
 
 def prompt_options(question: str, options: List[Tuple[str, str]]) -> str:
+    """
+    Prompts the user for one of a given set of options
+
+    args:
+        question: The question to display prior to the prompt
+        options: The list of possible option and description pairs
+
+    raises:
+        EOFError: If the user enters an EOF
+
+    returns:
+        The string entered by the user, which will correspond exactly to one of the options
+    """
     print(question)
+    # extra 2 for colon and an extra space
+    max_option_width = max(len(option) for option, _ in options) + 2
     for option, desc in options:
-        print(option + ":", desc)
+        print(
+            option + ":",
+            fill(
+                desc,
+                width=80,
+                initial_indent=" " * (max_option_width - len(option) - 2),
+                subsequent_indent=" " * max_option_width,
+            ),
+        )
     sys.stdout.write("[{}]: ".format("/".join([option for option, _ in options])))
     option_set = {option for option, _ in options}
     while True:
         result = input().strip()
         if result in option_set:
             return result
-        else:
-            sys.stdout.write(
-                l10n(
-                    "prompt-invalid-response-multiple",
-                    options="/".join([option for option, _ in options]),
-                )
+
+        sys.stdout.write(
+            l10n(
+                "prompt-invalid-response-multiple",
+                options="/".join([option for option, _ in options]),
             )
+        )
 
 
-def parse_num_list(string: str, max_val: int):
+def _parse_num_list(string: str, max_val: int) -> List[int]:
     if string == "":
-        return list()
+        return []
 
-    m = re.match(r"(\-?\d+)(?:-(\d+))?$", string)
-    if not m:
+    match = re.match(r"(\-?\d+)(?:-(\d+))?$", string)
+    if not match:
         raise ValueError(l10n("prompt-invalid-range", max=max_val))
-    start = m.group(1)
-    end = m.group(2) or start
+    start = match.group(1)
+    end = match.group(2) or start
     return list(range(int(start, 10), int(end, 10) + 1))
 
 
-def prompt_num_multi(question: str, max_val: int, cancel: bool = False):
-    print("{}: ".format(question))
+def prompt_num_multi(question: str, max_val: int, cancel: bool = False) -> List[int]:
+    """
+    Prompts the user for one or more numbers
+
+    Numbers must be positive, and must be separated by whitespace or commas.
+    Ranges can be indicated using a hyphen
+
+    args:
+        question: The question to display prior to the prompt
+        max_val: The maximum valid number that will be accepted
+        cancel: If true, the user has the option to enter -1 to raise an EOFError
+
+    raises:
+        EOFError: If the user enters -1 (and cancel is True) or EOF
+
+    returns:
+        The list of numbers the user entered, with ranges resolved into the component integers
+    """
+    print(f"{question}: ")
     while True:
         try:
-            result = [y for x in input().split(",") for y in parse_num_list(x, max_val)]
+            result = [
+                y
+                for x in re.split("[ ,]+", input().strip())
+                for y in _parse_num_list(x, max_val)
+            ]
             if -1 in result and cancel:
                 raise EOFError()
             if next(filter(lambda x: x > max_val or x < 0, result), None):
@@ -137,14 +210,31 @@ def prompt_num_multi(question: str, max_val: int, cancel: bool = False):
             print(l10n("prompt-invalid-range-multi", max=max_val))
 
 
-def prompt_num(question: str, max_val: int, cancel: bool = False):
+def prompt_num(question: str, max_val: int, cancel: bool = False) -> int:
+    """
+    Prompts the user for exactly one number
+
+    The input number must be positive
+
+    args:
+        question: The question to display prior to the prompt
+        max_val: The maximum valid number that will be accepted
+        cancel: If true, the user has the option to enter -1 to raise an EOFError
+
+    raises:
+        EOFError: If the user enters -1 (and cancel is True) or EOF
+
+    returns:
+        The number the user entered
+    """
+
     print("{}: ".format(question))
     while True:
         try:
-            result = int(input())
+            result = int(input().strip())
             if result > max_val or result < 0:
                 if result == -1 and cancel:
-                    return result
+                    raise EOFError()
 
                 print(l10n("prompt-range-too-large-number", max=max_val))
             else:
@@ -153,23 +243,32 @@ def prompt_num(question: str, max_val: int, cancel: bool = False):
             print(l10n("prompt-invalid-range", max=max_val))
 
 
-def prompt_str(question: str, default: Optional[str] = None) -> Optional[str]:
+def prompt_str(question: str, default: Optional[str] = None) -> str:
     """
-    Prompts for string input
+    Prompts the user for string input
+
+    args:
+        question: The question to display prior to the prompt
+        default: The default value to be used if the user provides no input
+
+    raises:
+        EOFError: If the user enters -1 (and cancel is True) or EOF
+
+    returns:
+        The value the user entered (stripped of leading and trailing whitespace),
+        or default if default is not None
 
-    If the user enters an EOF, returns None
     """
     if default:
         print(f'{question} (default "{default}"): ', end="")
     else:
         print(f"{question}: ", end="")
+
     while True:
-        try:
-            result = input()
-        except EOFError:
-            return None
+        result = input().strip()
 
         if result:
             return result
 
-        return default
+        if default is not None:
+            return default
