@@ -29,19 +29,17 @@ import urllib
 import urllib.parse
 from typing import AbstractSet, Dict, Iterable, List, Optional, Set, cast
 
-from portmod.config import get_config
 from portmod.globals import env
 from portmod.parsers.manifest import Manifest
 from portmod.repo import get_repo_name, get_repo_root
 from portmod.source import SourceManifest
+from portmodlib._deprecated import File, InstallDir
 from portmodlib.atom import Atom, FQAtom, QualifiedAtom
 from portmodlib.colour import green
 from portmodlib.pybuild import (
     BasePybuild,
-    File,
     FullInstalledPybuild,
     FullPybuild,
-    InstallDir,
     get_installed_env,
 )
 from portmodlib.source import Source, get_archive_basename
@@ -57,9 +55,11 @@ class Pybuild(BasePybuild):
         # Note: mypy doesn't like how we coerce INSTALL_DIRS
         if cache:
             self.__dict__ = cache
-            self.INSTALL_DIRS = [
-                InstallDir(**cast(Dict, idir)) for idir in self.INSTALL_DIRS
-            ]
+            if self._PYBUILD_VER == 1:
+                self.INSTALL_DIRS = [
+                    InstallDir(**cast(Dict, idir)) for idir in self.INSTALL_DIRS  # type: ignore
+                ]
+            self.TAGS = set(self.TAGS)
         for keyword, value in kwargs.items():
             setattr(self, keyword, value)
         self.ATOM = atom
@@ -72,7 +72,6 @@ class Pybuild(BasePybuild):
         self.PVR = atom.PVR
         self.CPN = QualifiedAtom(atom.CPN)
         self.CP = QualifiedAtom(atom.CP)
-        self.__ENV = None
         self.INSTALLED = False
         self.FILE = FILE
         self.REPO_PATH = get_repo_root(self.FILE)
@@ -182,18 +181,9 @@ class Pybuild(BasePybuild):
             return get_archive_basename(tmp_source.name)
         return None
 
-    def _get_install_dir_dest(self):
-        install_dir_dest = get_config().get("INSTALL_DEST", ".")
-        for attr in dir(self):
-            if not attr.startswith("_") and isinstance(getattr(self, attr), str):
-                install_dir_dest = install_dir_dest.replace(
-                    "{" + attr + "}", getattr(self, attr)
-                )
-        return os.path.normpath(install_dir_dest)
-
     def validate(self):
         """QA Checks pybuild structure"""
-        from portmod.repo.loader import pkg_exists
+        from portmod.repo.loader import _safe_load_file, pkg_exists
         from portmod.repo.metadata import (
             check_use_expand_flag,
             get_global_use,
@@ -211,9 +201,6 @@ class Pybuild(BasePybuild):
         if not isinstance(self.SRC_URI, str):
             raise TypeError("SRC_URI must be a string")
 
-        if not isinstance(self.DATA_OVERRIDES, str):
-            raise TypeError("DATA_OVERRIDES must be a string")
-
         if not isinstance(self.LICENSE, str):
             raise TypeError(
                 "LICENSE must be a string containing a space separated list of licenses"
@@ -253,7 +240,6 @@ class Pybuild(BasePybuild):
 
         rdeps = check_use_reduce("RDEPEND", token_class=Atom)
         deps = check_use_reduce("DEPEND", token_class=Atom)
-        overrides = check_use_reduce("DATA_OVERRIDES", token_class=Atom)
         check_use_reduce("HOMEPAGE")
         check_use_reduce("SRC_URI", is_src_uri=True)
         check_use_reduce("PATCHES")
@@ -271,53 +257,60 @@ class Pybuild(BasePybuild):
             if isinstance(atom, Atom) and not pkg_exists(atom, repo_name=self.REPO):
                 errors.append(f"Dependency {atom} could not be found!")
 
-        for atom in overrides:
-            if isinstance(atom, Atom) and not pkg_exists(atom, repo_name=self.REPO):
-                errors.append(f"Data Override {atom} could not be found!")
+        if self._PYBUILD_VER == 1:
+            if not isinstance(self.DATA_OVERRIDES, str):  # type: ignore # pylint: disable=no-member
+                errors.append("DATA_OVERRIDES must be a string")
+            else:
+                overrides = check_use_reduce("DATA_OVERRIDES", token_class=Atom)
+                for atom in overrides:
+                    if isinstance(atom, Atom) and not pkg_exists(
+                        atom, repo_name=self.REPO
+                    ):
+                        errors.append(f"Data Override {atom} could not be found!")
 
-        for install in self.INSTALL_DIRS:
-            if not isinstance(install, InstallDir):
-                errors.append(f'InstallDir "{install}" must have type InstallDir')
-                continue
-            for file in install.get_files():
-                if not isinstance(file, File):
-                    errors.append(f'File "{file}" must have type File')
+            for install in self.INSTALL_DIRS:
+                if not isinstance(install, InstallDir):
+                    errors.append(f'InstallDir "{install}" must have type InstallDir')
                     continue
+                for file in install.get_files():
+                    if not isinstance(file, File):
+                        errors.append(f'File "{file}" must have type File')
+                        continue
+
+                    try:
+                        check_required_use(file.REQUIRED_USE, set(), self.valid_use)
+                    except UseParserError as error:
+                        errors.append(f"Error processing file {file.NAME}: {error}")
 
                 try:
-                    check_required_use(file.REQUIRED_USE, set(), self.valid_use)
+                    check_required_use(install.REQUIRED_USE, set(), self.valid_use)
                 except UseParserError as error:
-                    errors.append(f"Error processing file {file.NAME}: {error}")
+                    errors.append(f"Error processing dir {install.PATH}: {error}")
 
-            try:
-                check_required_use(install.REQUIRED_USE, set(), self.valid_use)
-            except UseParserError as error:
-                errors.append(f"Error processing dir {install.PATH}: {error}")
+                if install.WHITELIST is not None and not isinstance(
+                    install.WHITELIST, list
+                ):
+                    errors.append(f"WHITELIST {install.WHITELIST} must be a list")
+                elif install.WHITELIST is not None:
+                    for string in install.WHITELIST:
+                        if not isinstance(string, str):
+                            errors.append(
+                                f'"{string}" in InstallDir WHITELIST is not a string'
+                            )
 
-            if install.WHITELIST is not None and not isinstance(
-                install.WHITELIST, list
-            ):
-                errors.append(f"WHITELIST {install.WHITELIST} must be a list")
-            elif install.WHITELIST is not None:
-                for string in install.WHITELIST:
-                    if not isinstance(string, str):
-                        errors.append(
-                            f'"{string}" in InstallDir WHITELIST is not a string'
-                        )
+                if install.BLACKLIST is not None and not isinstance(
+                    install.BLACKLIST, list
+                ):
+                    errors.append(f"BLACKLIST {install.BLACKLIST} must be a list")
+                elif install.BLACKLIST is not None:
+                    for string in install.BLACKLIST:
+                        if not isinstance(string, str):
+                            errors.append(
+                                f'"{string}" in InstallDir BLACKLIST is not a string'
+                            )
 
-            if install.BLACKLIST is not None and not isinstance(
-                install.BLACKLIST, list
-            ):
-                errors.append(f"BLACKLIST {install.BLACKLIST} must be a list")
-            elif install.BLACKLIST is not None:
-                for string in install.BLACKLIST:
-                    if not isinstance(string, str):
-                        errors.append(
-                            f'"{string}" in InstallDir BLACKLIST is not a string'
-                        )
-
-            if install.WHITELIST is not None and install.BLACKLIST is not None:
-                errors.append("WHITELIST and BLACKLIST are mutually exclusive")
+                if install.WHITELIST is not None and install.BLACKLIST is not None:
+                    errors.append("WHITELIST and BLACKLIST are mutually exclusive")
 
         global_use = get_global_use(self.REPO_PATH)
         metadata = get_package_metadata(self)
@@ -356,23 +349,39 @@ class Pybuild(BasePybuild):
         if not isinstance(self.HOMEPAGE, str) or "FILLME" in self.HOMEPAGE:
             errors.append("Please fill in the HOMEPAGE field")
 
-        all_sources = self._get_sources(matchall=True)
+        if self._PYBUILD_VER == 1:
+            try:
+                all_sources = self._get_sources(matchall=True)
+            except Exception as e:
+                all_sources = []
+                errors.append(str(e))
 
-        for install in self.INSTALL_DIRS:
-            if isinstance(install, InstallDir):
-                if len(all_sources) > 0 and install.S is None:
-                    if len(all_sources) != 1:
-                        errors.append(
-                            "InstallDir does not declare a source name but source "
-                            "cannot be set automatically"
-                        )
-            else:
-                errors.append(f"InstallDir {install} should be of type InstallDir")
+            for install in self.INSTALL_DIRS:
+                if isinstance(install, InstallDir):
+                    if len(all_sources) > 0 and install.S is None:
+                        if len(all_sources) != 1:
+                            errors.append(
+                                "InstallDir does not declare a source name but source "
+                                "cannot be set automatically"
+                            )
+                else:
+                    errors.append(f"InstallDir {install} should be of type InstallDir")
 
         manifest = self.get_manifest()
-        for source in self._get_sources(matchall=True):
-            if manifest.get(source.name) is None:
-                errors.append(f'Source "{source.name}" is not listed in the Manifest')
+        try:
+            for source in self._get_sources(matchall=True):
+                if manifest.get(source.name) is None:
+                    errors.append(
+                        f'Source "{source.name}" is not listed in the Manifest'
+                    )
+        except UseParserError as e:
+            errors.append(str(e))
+
+        if "validate" in self.FUNCTIONS:
+            # Run the validate function in the RestrictedPython Sandbox.
+            # This allows subclasses in the packaging environment to do their own validation
+            pkg = _safe_load_file(self.FILE, installed=self.INSTALLED)
+            errors.extend(pkg.validate())
 
         if len(errors) > 0:
             raise Exception(
@@ -476,10 +485,18 @@ def to_cache(pkg: FullPybuild) -> Dict:
         "DATA_OVERRIDES",
         "S",
         "PATCHES",
+        "_PYBUILD_VER",
+        "TAGS",
     ]:
-        cache[key] = getattr(pkg, key)
+        if hasattr(pkg, key):
+            cache[key] = getattr(pkg, key)
 
-    cache["INSTALL_DIRS"] = [idir._to_cache() for idir in pkg.INSTALL_DIRS]
+    if pkg._PYBUILD_VER == 1:
+        cache["INSTALL_DIRS"] = [
+            idir._to_cache()
+            for idir in getattr(pkg, "INSTALL_DIRS")
+            if isinstance(idir, InstallDir)
+        ]
     phase_functions = [
         "src_unpack",
         "src_install",
@@ -488,6 +505,8 @@ def to_cache(pkg: FullPybuild) -> Dict:
         "pkg_pretend",
         "pkg_postinst",
         "pkg_prerm",
+        # Not actually a phase function
+        "validate",
     ]
     cache["FUNCTIONS"] = [
         func
