@@ -5,8 +5,8 @@
 Module for performing bulk queries on the mod database and repositories
 """
 
-import json
 import logging
+import multiprocessing
 import os
 import re
 import shutil
@@ -14,11 +14,10 @@ import sys
 from collections import defaultdict
 from functools import cmp_to_key
 from logging import info
+from multiprocessing import Process
 from time import sleep
-from types import SimpleNamespace
 from typing import (
     AbstractSet,
-    Any,
     DefaultDict,
     Dict,
     Iterable,
@@ -29,9 +28,12 @@ from typing import (
     Union,
 )
 
+from progressbar import AnimatedMarker, ProgressBar, UnknownLength, Variable
+
 from portmodlib.atom import Atom, FQAtom, QualifiedAtom, atom_sat, version_gt
 from portmodlib.colour import blue, bright, cyan, green, lblue, lgreen, red, yellow
 from portmodlib.l10n import l10n
+from portmodlib.portmod import Group, PackageIndexData, Person
 from portmodlib.portmod import query as native_query
 from portmodlib.portmod import update_index as native_update_index
 from portmodlib.usestr import parse_usestr
@@ -50,45 +52,32 @@ from .repo.metadata import get_package_metadata as get_native_metadata
 from .repo.metadata import get_use_expand_values
 from .util import get_newest, sort_by_version
 
-
-class PackageMetadata(SimpleNamespace):
-    """Data class for package metadata returned by query results"""
-
-    cpn: str
-    prefix: str
-    category: str
-    package: str
-    name: str
-    desc: str
-    homepage: Optional[str] = None
-    other_homepages: Optional[str] = None
-    license: Optional[str] = None
-    longdescription: Optional[str] = None
-    maintainer: Optional[str] = None
-    upstream_maintainer: Optional[str] = None
-    upstream_doc: Optional[str] = None
-    upstream_bugs_to: Optional[str] = None
-    upstream_changelog: Optional[str] = None
+Maintainer = Union[Group, Person]
+Maintainers = Union[Maintainer, List[Maintainer]]
 
 
-def get_maintainer_string(maintainers: Union[List[Any], Any]) -> str:
-    def list_maintainers_to_human_strings(maintainers: List[Any]) -> str:
-        """return the list of maintainers as a human readible string"""
-        result = ""
-        for maintainer_id in range(len(maintainers)):
-            maintainer = str(maintainers[maintainer_id])
-            if maintainer_id >= len(maintainers) - 1:  # the last
-                result += maintainer
-            elif maintainer_id >= len(maintainers) - 2:  # the second last
-                result += maintainer + " and "
-            else:
-                result += maintainer + ", "
-        return result
-
+def get_maintainer_strings(maintainers: Maintainers) -> List[str]:
     if not isinstance(maintainers, list):
         maintainers = [maintainers]
+    return [str(maintainer) for maintainer in maintainers]
 
-    return list_maintainers_to_human_strings(maintainers)
+
+def get_maintainer_string(maintainers: Maintainers) -> str:
+    return list_maintainers_to_human_strings(get_maintainer_strings(maintainers))
+
+
+def list_maintainers_to_human_strings(maintainers: List[str]) -> str:
+    """return the list of maintainers as a human readible string"""
+    result = ""
+    for maintainer_id in range(len(maintainers)):
+        maintainer = maintainers[maintainer_id]
+        if maintainer_id >= len(maintainers) - 1:  # the last
+            result += maintainer
+        elif maintainer_id >= len(maintainers) - 2:  # the second last
+            result += maintainer + " and "
+        else:
+            result += maintainer + ", "
+    return result
 
 
 def print_depgraph(
@@ -164,22 +153,13 @@ def str_squelch_sep(value: str) -> str:
 def query(
     value: str,
     limit: int = 10,
-) -> List[PackageMetadata]:
+) -> List[PackageIndexData]:
     """
     Finds mods that contain the given value in the given field
     """
-    results = []
     if not env.PREFIX_NAME:
         raise RuntimeError("Queries must be done inside a prefix!")
-    for result in native_query(env.INDEX, env.PREFIX_NAME, value, limit):
-        data = json.loads(result)
-        # Results from tantivy are all lists?
-        metadata = {}
-        for key, value in data.items():
-            metadata[key] = value[0]
-        results.append(PackageMetadata(**metadata))
-
-    return results
+    return native_query(env.INDEX, env.PREFIX_NAME, value, limit)
 
 
 # FIXME: For this to be indexed, we'll need to add DEPEND and RDEPEND to the index
@@ -270,7 +250,7 @@ def get_flag_string(
 
 
 def display_search_results(
-    packages: Sequence[PackageMetadata],
+    packages: Sequence[PackageIndexData],
     summarize: bool = True,
     numbers: bool = False,
     file=sys.stdout,
@@ -279,6 +259,17 @@ def display_search_results(
     Prettily formats a list of package metadata for use in search results
     """
     count = 0
+
+    footnotes: Dict[str, str] = {}
+    footnotenum = "a"
+
+    def footnote(repo: str):
+        nonlocal footnotenum
+        if repo not in footnotes:
+            footnotes[repo] = str(footnotenum)
+            footnotenum = chr(ord(footnotenum) + 1)
+        return lblue(f"[{footnotes[repo]}]")
+
     for package in packages:
         count += 1
         group = load_pkg(Atom(package.cpn))
@@ -288,7 +279,7 @@ def display_search_results(
         download = get_total_download_size([newest])
 
         if installed is not None:
-            installed_str = blue(bright(installed.PV))
+            installed_str = blue(bright(installed.PVR)) + footnote(installed.REPO)
 
             flags = {flag.lstrip("+") for flag in installed.IUSE if "_" not in flag}
             usestr = get_flag_string(
@@ -324,13 +315,11 @@ def display_search_results(
                     string = get_flag_string(use, enabled_expand, disabled_expand, None)
                     use_expand_strings.append(string)
 
-            installed_str += (
-                " {"
-                + " ".join(
-                    list(filter(None, [usestr, texture_string] + use_expand_strings))
-                )
-                + "}"
+            all_flags: List[str] = list(
+                filter(None, [usestr, texture_string] + use_expand_strings)
             )
+            if all_flags:
+                installed_str += " {" + " ".join(all_flags) + "}"
         else:
             installed_str = "not installed"
 
@@ -340,20 +329,25 @@ def display_search_results(
         versions = []
         ARCH = env.prefix().ARCH
         for mod in sortedmods:
+            if mod.INSTALLED:
+                continue
+
             if ARCH in mod.KEYWORDS:
-                version_nofmt = mod.PV
-                version = green(mod.PV)
+                version_nofmt = mod.PVR
+                version = green(mod.PVR)
             elif "~" + ARCH in mod.KEYWORDS:
-                version_nofmt = "(~)" + mod.PV
+                version_nofmt = "(~)" + mod.PVR
                 version = yellow(version_nofmt)
             else:
-                version_nofmt = "**" + mod.PV
+                version_nofmt = "**" + mod.PVR
                 version = red(version_nofmt)
 
+            repo = mod.REPO
+
             if is_masked(mod.ATOM, mod.REPO):
-                versions.append(red("[M]" + version_nofmt))
+                versions.append(red("[M]" + version_nofmt) + footnote(repo))
             else:
-                versions.append(version)
+                versions.append(version + footnote(repo))
         version_str = " ".join(versions)
         flags = {
             flag.lstrip("+") for mod in group for flag in mod.IUSE if "_" not in flag
@@ -381,13 +375,9 @@ def display_search_results(
                 string = get_flag_string(use, [], flags, None, display_minuses=False)
                 use_expand_strings.append(string)
 
-        version_str += (
-            " {"
-            + " ".join(
-                list(filter(None, [usestr, texture_string] + use_expand_strings))
-            )
-            + "}"
-        )
+        all_flags = list(filter(None, [usestr, texture_string] + use_expand_strings))
+        if all_flags:
+            version_str += " {" + " ".join(all_flags) + "}"
 
         # If there are multiple URLs, remove any formatting from the pybuild and
         # add padding
@@ -395,77 +385,59 @@ def display_search_results(
         if logging.root.level <= logging.DEBUG:
             if package.homepage:
                 formatted_homepages = "\n                 ".join(
-                    [package.homepage] + (package.other_homepages or "").split()
+                    [package.homepage] + package.other_homepages
                 )
 
                 homepage_str = (
-                    "       {} {}".format(
-                        green(l10n("package-homepage")), formatted_homepages
-                    ),
+                    f"       {green(l10n('package-homepage'))} {formatted_homepages}"
                 )
             print(
-                "{} {}  {}".format(
-                    cyan(count) if numbers else "", green("*"), bright(package.cpn)
-                ),
-                "       {} {}".format(green(l10n("package-name")), package.name),
-                "       {} {}".format(
-                    green(l10n("package-available-versions")), version_str
-                ),
-                "       {} {}".format(
-                    green(l10n("package-installed-version")), installed_str
-                ),
-                "       {} {}".format(green(l10n("package-size-of-files")), download),
+                f"{cyan(count) if numbers else ''} {green('*')}  {bright(package.cpn)}",
+                f"       {green(l10n('package-name'))} {package.name}",
+                f"       {green(l10n('package-available-versions'))} {version_str}",
+                f"       {green(l10n('package-installed-version'))} {installed_str}",
+                f"       {green(l10n('package-size-of-files'))} {download}",
                 sep=os.linesep,
                 file=file,
             )
             if package.homepage:
                 print(homepage_str, file=file)
             print(
-                "       {} {}".format(
-                    green(l10n("package-description")), str_squelch_sep(newest.DESC)
-                ),
-                "       {} {}".format(green(l10n("package-license")), newest.LICENSE),
+                f"       {green(l10n('package-description'))} {str_squelch_sep(newest.DESC)}",
+                f"       {green(l10n('package-license'))} {newest.LICENSE}",
                 sep=os.linesep,
                 file=file,
             )
-            if package.upstream_maintainer:
+            if package.upstream_maintainers:
                 print(
-                    "       {} {}".format(
-                        green(l10n("package-upstream-author")),
-                        package.upstream_maintainer,
-                    ),
+                    f"       {green(l10n('package-upstream-author'))} {list_maintainers_to_human_strings(package.upstream_maintainers)}",
                     file=file,
                 )
             print(file=file)
         else:
             print(
-                "{} {}  {}".format(
-                    cyan(count) if numbers else "", green("*"), bright(package.cpn)
-                ),
-                "       {} {}".format(
-                    green(l10n("package-available-versions")), version_str
-                ),
-                "       {} {}".format(
-                    green(l10n("package-installed-version")), installed_str
-                ),
+                f"{cyan(count) if numbers else ''} {green('*')}  {bright(package.cpn)}",
+                f"       {green(l10n('package-available-versions'))} {version_str}",
+                f"       {green(l10n('package-installed-version'))} {installed_str}",
                 sep=os.linesep,
                 file=file,
             )
             if package.homepage:
                 print(
-                    "       {} {}".format(
-                        green(l10n("package-homepage")), package.homepage
-                    ),
+                    f"       {green(l10n('package-homepage'))} {package.homepage}",
                     file=file,
                 )
             print(
-                "       {} {}".format(
-                    green(l10n("package-description")), str_squelch_sep(newest.DESC)
-                ),
+                f"       {green(l10n('package-description'))} {str_squelch_sep(newest.DESC)}",
                 sep=os.linesep,
                 file=file,
             )
 
+    if footnotes:
+        print()
+        for repo in footnotes:
+            print(footnote(repo), f'"{repo}"', get_repo(repo).location)
+
     if summarize:
         print("\n" + l10n("packages-found", num=len(packages)), file=file)
 
@@ -569,7 +541,7 @@ def get_flags(
     return local_flags, global_flags, use_expand_flags
 
 
-def get_package_metadata(package: QualifiedAtom) -> Optional[PackageMetadata]:
+def get_package_metadata(package: QualifiedAtom) -> Optional[PackageIndexData]:
     packages = load_pkg(package)
     if not packages:
         return None
@@ -583,7 +555,9 @@ def get_package_metadata(package: QualifiedAtom) -> Optional[PackageMetadata]:
     pkgs = sorted(packages, key=cmp_to_key(version_cmp))
     newest = pkgs[-1]
 
-    pkg_metadata = PackageMetadata(
+    assert env.PREFIX_NAME, "A prefix must be specified"
+
+    pkg_metadata = PackageIndexData(
         cpn=package.CPN,
         prefix=env.PREFIX_NAME,
         category=package.C,
@@ -597,21 +571,22 @@ def get_package_metadata(package: QualifiedAtom) -> Optional[PackageMetadata]:
     if homepages:
         pkg_metadata.homepage = homepages[0]
     if len(homepages) > 1:
-        pkg_metadata.other_homepages = " ".join(homepages[1:])
+        pkg_metadata.other_homepages = homepages[1:]
 
     if newest.LICENSE:
         pkg_metadata.license = newest.LICENSE
 
     for pkg in pkgs:
+        pkg_metadata.tags |= pkg.TAGS
         metadata = get_native_metadata(pkg)
         if metadata:
             if metadata.longdescription:
                 pkg_metadata.longdescription = metadata.longdescription
             if metadata.maintainer:
-                pkg_metadata.maintainer = get_maintainer_string(metadata.maintainer)
+                pkg_metadata.maintainers = get_maintainer_strings(metadata.maintainer)
             if metadata.upstream:
                 if metadata.upstream.maintainer:
-                    pkg_metadata.upstream_maintainer = get_maintainer_string(
+                    pkg_metadata.upstream_maintainers = get_maintainer_strings(
                         metadata.upstream.maintainer
                     )
                 if metadata.upstream.doc:
@@ -624,7 +599,7 @@ def get_package_metadata(package: QualifiedAtom) -> Optional[PackageMetadata]:
     return pkg_metadata
 
 
-def _get_index_data() -> List[PackageMetadata]:
+def _get_index_data() -> List[PackageIndexData]:
     if not env.PREFIX_NAME:
         return []
 
@@ -640,7 +615,7 @@ def _get_index_data() -> List[PackageMetadata]:
                 yield pkg.CPN
 
     info("Beginning index update for prefix " + env.PREFIX_NAME)
-    if sys.stderr.isatty():
+    if sys.stderr.isatty() or env.TESTING:
         try:
             from progressbar import GranularBar as Bar
         except ImportError:
@@ -697,39 +672,37 @@ def _get_index_data() -> List[PackageMetadata]:
     return package_data
 
 
-def _commit_index(package_data: List[PackageMetadata]):
-    from multiprocessing import Process
+def _run_progress_bar(pipe):
+    bar = ProgressBar(
+        widgets=[
+            Variable("status", format="{variables.status}"),
+            " ",
+            AnimatedMarker(),
+        ],
+        max_value=UnknownLength,
+        variables={"status": "Updating index"},
+    )
+    bar.start()
+    while not pipe.poll():
+        bar.update()
+        sleep(0.1)
+    bar.update(status="Done updating index.")
+    bar.finish()
 
-    from progressbar import AnimatedMarker, ProgressBar, UnknownLength, Variable
 
+def _commit_index(package_data: List[PackageIndexData]):
     os.makedirs(env.INDEX, exist_ok=True)
 
-    if sys.stderr.isatty():
-        bar = ProgressBar(
-            widgets=[
-                Variable("status", format="{variables.status}"),
-                " ",
-                AnimatedMarker(),
-            ],
-            max_value=UnknownLength,
-            variables={"status": "Updating index"},
-        )
-        bar.start()
-
-        process = Process(
-            target=native_update_index,
-            args=(env.INDEX, [pkg.__dict__ for pkg in package_data]),
-        )
+    if sys.stderr.isatty() or env.TESTING:
+        pipe, child_pipe = multiprocessing.Pipe()
+        process = Process(target=_run_progress_bar, args=(child_pipe,))
         process.start()
-        while process.is_alive():
-            bar.update()
-            sleep(0.1)
-
-        bar.update(status="Done updating index.")
-        bar.finish()
+        native_update_index(env.INDEX, package_data)
+        pipe.send(True)
+        process.join()
     else:
         info("Updating index...")
-        native_update_index(env.INDEX, [pkg.__dict__ for pkg in package_data])
+        native_update_index(env.INDEX, package_data)
         info("Done updating index.")
 
 
