@@ -19,8 +19,8 @@ from portmod.functools import install_cache, prefix_aware_cache, system_cache
 from portmod.globals import env
 from portmod.lock import vdb_lock
 from portmod.parsers.manifest import Manifest
-from portmod.repo import Repo, get_repo, get_repo_name
-from portmod.repo.metadata import get_categories, get_masters
+from portmod.repo import Repo, get_repo, get_repo_name, get_repo_root
+from portmod.repo.metadata import get_categories, get_masters, is_pybuild_version_banned
 from portmod.util import get_max_version
 from portmod.vdb import VDB, vdb_path
 from portmodlib._loader import (
@@ -31,7 +31,7 @@ from portmodlib._loader import (
 )
 from portmodlib.atom import Atom, FQAtom, QualifiedAtom, VAtom, atom_sat
 from portmodlib.l10n import l10n
-from portmodlib.pybuild import File, FullPybuild, InstallDir
+from portmodlib.pybuild import FullPybuild
 
 from .updates import get_moved
 
@@ -205,12 +205,24 @@ def _import_common(
         result = load_function(path, installed=True, repo=None)
     else:
         versions = {}
-        for file in _iterate_pybuilds(base_atom, repo_name=repo):
+
+        def get_version(file: str, allow_masked: bool):
             basename, _ = os.path.splitext(os.path.basename(file))
             atom = VAtom("common/" + basename)
-            if not is_masked(atom, repo):
+            if not is_masked(atom, repo) or allow_masked:
                 versions[atom.PVR] = file
 
+        files = list(_iterate_pybuilds(base_atom, repo_name=repo))
+
+        for file in files:
+            get_version(file, False)
+
+        if not versions:
+            # If no versions were found, then try loading masked versions
+            # Masked packages will still fail dependency resolution,
+            for file in files:
+                get_version(file, True)
+
         max_version = get_max_version(versions.keys())
         if not max_version:
             raise Exception(f"Could not find package {name}")
@@ -222,7 +234,7 @@ def _import_common(
     )
 
 
-def __safe_load_file(path: str, *, installed=False) -> FullPybuild:
+def _safe_load_file(path: str, *, installed=False) -> FullPybuild:
     """
     Loads a pybuild file
 
@@ -278,6 +290,8 @@ def __safe_load_module(path: str, installed: bool, repo: Optional[str]):
         # to prevent modification of class attributes.
         # the instances are safe as long as underscored attributes are inaccessible
 
+        from portmodlib._deprecated import File, InstallDir
+
         def file_func(*args, **kwargs):
             return File(*args, **kwargs)
 
@@ -285,17 +299,32 @@ def __safe_load_module(path: str, installed: bool, repo: Optional[str]):
             return InstallDir(*args, **kwargs)
 
         # create simplified version of Pybuild to use for inheritance.
-        # It has default values for all the pyubild fields.
+        # It has default values for all the pybuild fields.
         # We don't use FullPybuild directly to prevent its fields being modified
         # This class is generated within a function, and thus will not be the same
         # for any two packages.
-        class Pybuild1(FullPybuild):
-            """Generated Pybuild1 for use in non-sandboxed loader"""
+        class Pybuild2(FullPybuild):
+            """Generated Pybuild2 for use in non-sandboxed loader"""
 
-        return {
+            _PYBUILD_VER = 2
+
+        class Pybuild1(FullPybuild):
+            """Generated Pybuild2 for use in non-sandboxed loader"""
+
+            _PYBUILD_VER = 1
+            DATA_OVERRIDES = ""
+            INSTALL_DIRS = []
+            REBUILD_FILES = []
+            TIER = "a"
+
+            def __init__(self):
+                super().__init__()
+                if type(self.TIER) is int:
+                    self.TIER = str(self.TIER)
+
+        imports = {
             "File": file_func,
             "InstallDir": install_dir_func,
-            "Pybuild1": Pybuild1,
             "DOWNLOAD_DIR": env.DOWNLOAD_DIR,
             "version_gt": None,
             "find_file": None,
@@ -306,6 +335,18 @@ def __safe_load_module(path: str, installed: bool, repo: Optional[str]):
             "apply_patch": None,
             "__name__": "pybuild",
         }
+        pybuild_versions = {
+            1: Pybuild1,
+            2: Pybuild2,
+        }
+
+        repo_root = get_repo_root(path)
+        for version, class_ in pybuild_versions.items():
+            # If the package is an installed package, or the version is not banned
+            if not repo_root or not is_pybuild_version_banned(repo_root, version):
+                imports[f"Pybuild{version}"] = class_
+
+        return imports
 
     def get_typing():
         import typing
@@ -321,27 +362,46 @@ def __safe_load_module(path: str, installed: bool, repo: Optional[str]):
     def _import(name, _globs, _loc, _fromlist, _level):
         if name.startswith("common."):
             return deepcopy(_import_common(name, installed, __safe_load_module, repo))
+        if name == "common":
+            raise RuntimeError(
+                "Cannot import from common directly. Import from a submodule instead"
+            )
         if name == "pybuild":
             return SimpleNamespace(**get_pybuild())
         if name == "pybuild.info":
             return SimpleNamespace(**get_info())
         if name == "typing":
             return SimpleNamespace(**get_typing())
-        if _fromlist:
-            return SimpleNamespace(**{item: None for item in _fromlist})
-        else:
 
-            class NoAttr:
-                def __getattr__(self, attr):
-                    # Necessary so that the object can be copied
-                    # Note that RestrictedPython prevents use of such attributes through ast parsing
-                    if attr.startswith("_"):
-                        return super().__getattr__(attr)  # type: ignore  # pylint: disable=no-member
-                    raise RuntimeError(
-                        f"Cannot use {name}.{attr}: Complex code in the global scope is not allowed!"
+        class NoAttr(type):
+            def __init__(cls, typename, bases, dictionary):
+                super().__init__(typename, bases, dictionary)
+                # Note: trailing space should mean that this error can't be ignored by subclassing this
+                # with a class named NoAttr. This probably isn't a security issue.
+                if cls.__name__ != "NoAttr ":
+                    raise TypeError(
+                        f"{typename} cannot inherit from {cls.item}. "
+                        "Only classes imported from `common.*` and `pybuild` can be inherited from"
                     )
 
-            return NoAttr()
+            def __getattr__(cls, attr):
+                # Necessary so that the object can be copied
+                # Note that RestrictedPython prevents use of such attributes through ast parsing
+                if attr.startswith("_"):
+                    return super().__getattr__(attr)  # type: ignore  # pylint: disable=no-member
+                raise RuntimeError(
+                    f"Cannot use {name}.{attr}: Complex code in the global scope is not allowed!"
+                )
+
+        if _fromlist:
+            return SimpleNamespace(
+                **{
+                    item: NoAttr("NoAttr ", (object,), {"item": item})
+                    for item in _fromlist
+                }
+            )
+        else:
+            return NoAttr("NoAttr ", (object,), {})
 
     filename, _ = os.path.splitext(os.path.basename(path))
 
