@@ -37,6 +37,20 @@ def __get_layout(repo_path: str) -> Dict:
     return {}
 
 
+@system_cache
+def is_pybuild_version_banned(repo_path: str, version: int) -> bool:
+    banned_versions = set(__get_layout(repo_path).get("pybuild_versions_banned", []))
+    return version in banned_versions
+
+
+@system_cache
+def is_pybuild_version_deprecated(repo_path: str, version: int) -> bool:
+    banned_versions = set(
+        __get_layout(repo_path).get("pybuild_versions_deprecated", [])
+    )
+    return version in banned_versions
+
+
 def get_master_names(repo_path: str) -> Set[str]:
     """Returns the direct masters for the repository at the given path"""
     masters = set()
