@@ -23,7 +23,7 @@ from portmodlib.usestr import use_reduce
 
 from .config import get_config
 from .config.sets import is_selected
-from .config.use import get_use, get_use_expand
+from .config.use import get_use
 from .download import fetchable, find_download, get_download_size
 from .loader import (
     SandboxedError,
@@ -166,52 +166,39 @@ class UseDep:
 def get_usestrings(
     mod: Pybuild,
     installed_use: Optional[Set[str]],
+    enabled_use: Set[str],
     verbose: bool,
-    transactions: Optional[Transactions] = None,
 ) -> List[str]:
-    def is_installed(atom: Atom):
-        if transactions and any(
-            atom_sat(trans.pkg.ATOM, atom)
-            for trans in transactions.pkgs
-            if not isinstance(trans, Delete)
-        ):
-            return True
-
-        if load_installed_pkg(atom):
-            return True
-
-        return False
-
-    enabled_use, disabled_use = get_use(mod, is_installed=is_installed)
-    if transactions:
-        for use in filter(lambda x: isinstance(x, UseDep), transactions.config):
-            if atom_sat(mod.ATOM, use.atom):
-                if use.flag.startswith("-"):
-                    enabled_use.discard(use.flag.lstrip("-"))
-                else:
-                    enabled_use.add(use.flag)
-
     # Note: flags containing underscores are USE_EXPAND flags
     # and are displayed separately
     IUSE_STRIP = {flag.lstrip("+") for flag in mod.IUSE if "_" not in flag}
 
     texture_options = use_reduce(
-        mod.TEXTURE_SIZES, enabled_use, disabled_use, flat=True, token_class=int
+        mod.TEXTURE_SIZES, enabled_use, flat=True, token_class=int
     )
 
     use_expand_strings = []
     for use in get_config().get("USE_EXPAND", []):
         if use in get_config().get("USE_EXPAND_HIDDEN", []):
             continue
+        prefix = use.lower() + "_"
 
-        enabled_expand, disabled_expand = get_use_expand(mod, use)
+        def base_flag(flag: str):
+            return re.sub(f"^{prefix}", "", flag)
+
+        enabled_expand = {
+            base_flag(flag) for flag in enabled_use if flag.startswith(prefix)
+        }
+        disabled_expand = {
+            base_flag(flag)
+            for flag in mod.IUSE_EFFECTIVE
+            if flag.startswith(prefix) and base_flag(flag) not in enabled_expand
+        }
         if enabled_expand or disabled_expand:
             installed_expand: Optional[Set[str]]
             if installed_use is not None:
                 installed_expand = {
-                    re.sub(f"^{use.lower()}_", "", flag)
-                    for flag in installed_use
-                    if flag.startswith(use.lower() + "_")
+                    base_flag(flag) for flag in installed_use if flag.startswith(prefix)
                 }
             else:
                 installed_expand = None
@@ -275,7 +262,7 @@ def print_transactions(
         if isinstance(trans, Delete):
             usestring = ""
         else:
-            usestrings = get_usestrings(pkg, installed_use, v, transactions)
+            usestrings = get_usestrings(pkg, installed_use, trans.flags, v)
             usestring = " ".join(list(filter(None, usestrings)))
 
         trans_colour = trans.__class__.COLOUR
@@ -360,7 +347,10 @@ def sort_transactions(transactions: Transactions):
             if isinstance(trans, Delete):
                 priorities[trans.pkg.ATOM] = "z"
             else:
-                priorities[trans.pkg.ATOM] = trans.pkg.TIER
+                if trans.pkg._PYBUILD_VER == 1:
+                    priorities[trans.pkg.ATOM] = trans.pkg.TIER  # type: ignore
+                else:
+                    priorities[trans.pkg.ATOM] = "a"
 
         def add_depends(mod, key: str, delete: bool):
             depends = {}
