@@ -30,7 +30,9 @@ class VDB(ContextDecorator):
         return self.gitrepo
 
     def __exit__(self, *exc):
-        if self.message is not None:
+        if self.message is not None and (
+            not self.gitrepo.heads or self.gitrepo.git.diff("HEAD", cached=True)
+        ):
             self.gitrepo.git.commit(m=self.message)
         self.lock.__exit__()
         return False
