@@ -13,6 +13,7 @@ import os
 import pickle
 import re
 import sys
+from traceback import extract_tb, format_exception_only, format_list
 from types import SimpleNamespace
 
 from ._loader import load_file, load_installed
@@ -25,6 +26,34 @@ from .pybuild import FullPybuild
 from .util import pybuild_dumper
 
 
+def excepthook(typ, value, tb):
+    """An exception hook which suppresses frames from this module in the traceback"""
+    if value.__cause__:
+        cause = value.__cause__
+        excepthook(type(cause), cause, cause.__traceback__)
+        print()
+        print(
+            "The above exception was the direct cause of the following exception:"
+            + os.linesep
+        )
+    if value.__context__:
+        context = value.__context__
+        excepthook(type(context), context, context.__traceback__)
+        print()
+        print(
+            "During handling of the above exception, another exception occurred:"
+            + os.linesep
+        )
+    show = [
+        fs
+        for fs in extract_tb(tb)
+        if fs.filename and fs.filename not in (__file__, "<string>")
+    ]
+    fmt = format_list(show) + format_exception_only(typ, value)
+    print("Traceback (most recent call last):")
+    print("".join(fmt), end="", file=sys.stderr)
+
+
 def wrapper_pybuild(args):
     # Set info variables
     from pybuild.info import _set_info
@@ -76,10 +105,10 @@ def wrapper_pybuild(args):
             sys.exit(int(result) * 142)
 
     if args.state_dir:
-        with open(os.path.join(args.state_dir, "state.pickle"), "wb") as b_file:
-            pickle.dump(pkg.__dict__, b_file)
+        with open(os.path.join(args.state_dir, "state.pickle"), "wb") as b_file_out:
+            pickle.dump(pkg.__dict__, b_file_out)
 
-        with open(os.path.join(args.state_dir, "environment.xz"), "wb") as b_file:
+        with open(os.path.join(args.state_dir, "environment.xz"), "wb") as b_file_out:
             # Keys are sorted to produce consistent results and
             # easy to read commits in the DB
             dictionary = pkg.__class__.__dict__.copy()
@@ -91,7 +120,7 @@ def wrapper_pybuild(args):
                 )
             )
             jsonstr = json.dumps(dictionary, default=pybuild_dumper, sort_keys=True)
-            b_file.write(lzma.compress(str.encode(jsonstr)))
+            b_file_out.write(lzma.compress(str.encode(jsonstr)))
 
 
 def wrapper_module(args):
@@ -200,4 +229,8 @@ def main():
     if args.verbosity is not None:
         logging.root.setLevel(args.verbosity)
 
+    # In debug-level verbosity or lower, use default full traces
+    # Otherwise suppress the parts referencing this module
+    if logging.root.level > logging.DEBUG:
+        sys.excepthook = excepthook
     args.func(args)
