@@ -11,7 +11,7 @@ def pybuild_dumper(obj):
     # Serialize as best we can. Sets become lists and unknown objects are
     # just stringified
     if isinstance(obj, set):
-        return list(obj)
+        return list(sorted(obj))
     if hasattr(obj, "to_json"):
         return obj.to_json()
     return "{}".format(obj)
