@@ -15,12 +15,47 @@ from typing import Optional
 from portmodlib.atom import VAtom
 
 CATEGORY: str
+"""
+The package's category.
+
+E.g. ``base``
+"""
 P: str
+"""
+The package name and version.
+
+E.g.: ``example-suite-1.0``
+"""
 PF: str
+"""
+The package name with version and revision.
+
+E.g.: ``example-suite-1.0-r1``
+"""
 PN: str
+"""
+The package name without version.
+
+E.g.: ``example-suite``
+"""
 PV: str
+"""
+The package's version without revision
+
+E.g. ``1.0``
+"""
 PR: Optional[str]
+"""
+The package's revision
+
+E.g. ``r1``
+"""
 PVR: str
+"""
+The package's version and revision
+
+E.g. ``1.0-r1``
+"""
 
 
 def _set_info(filename):
