@@ -28,19 +28,19 @@ def test_depclean(setup):
     """
     Tests that deselected mods are then depcleaned
     """
-    configure(["test/test-1.0", "test/test2-1.0"], no_confirm=True)
+    configure(["test/test-1.0", "test/test2-1.0"])
     mod = load_installed_pkg(Atom("test/test2"))
     assert mod
     assert mod in load_all_installed()
-    deselect(["test/test2"], no_confirm=True)
-    configure([], no_confirm=True, depclean=True)
+    deselect(["test/test2"])
+    configure([], depclean=True)
     assert not load_installed_pkg(Atom("test/test2"))
 
     mod = load_installed_pkg(Atom("test/test"))
     assert mod
     assert mod in load_all_installed()
-    deselect(["test/test"], no_confirm=True)
-    configure([], no_confirm=True, depclean=True)
+    deselect(["test/test"])
+    configure([], depclean=True)
     # Note: test/test is a system mod, so it cannot be removed
 
 
@@ -48,8 +48,8 @@ def test_noarg_depclean(setup):
     """
     Tests that deselected mods are then depcleaned
     """
-    configure(["test/test6-1.0"], no_confirm=True)
-    configure(["test/test6-1.0"], no_confirm=True, delete=True)
+    configure(["test/test6-1.0"])
+    configure(["test/test6-1.0"], delete=True)
     assert load_installed_pkg(Atom("test/test3"))
-    configure([], no_confirm=True, depclean=True)
+    configure([], depclean=True)
     assert not load_installed_pkg(Atom("test/test3"))
