@@ -7,7 +7,6 @@ import sys
 
 import pytest
 
-import pybuild
 from portmod._cli.pybuild import pybuild_manifest
 from portmod.cache import clear_cache_for_path
 from portmod.globals import env
@@ -15,7 +14,7 @@ from portmod.loader import SandboxedError, _sandbox_execute_pybuild, load_file
 from portmod.package import install_pkg, src_prepare, src_unpack
 from portmod.perms import Permissions
 from portmod.repo import Repo
-from portmod.repo.loader import __safe_load_file
+from portmod.repo.loader import _safe_load_file
 from portmod.win32 import get_personal
 
 from .env import TEST_REPO_DIR, rmtree, setup_env, tear_down_env
@@ -85,7 +84,7 @@ def create_pybuild(filestring, manifest: bool = False):
 
 
 def full_load_file(file: str):
-    return __safe_load_file(file)
+    return _safe_load_file(file)
 
 
 def test_safe():
@@ -99,7 +98,7 @@ class Package(Pybuild1):
     LICENSE="GPL-3"
 """
     create_pybuild(file)
-    __safe_load_file(TMP_FILE)
+    _safe_load_file(TMP_FILE)
 
 
 def test_write_globalscope():
@@ -320,7 +319,7 @@ class Package(Pybuild1):
 """
     with pytest.raises(NotImplementedError):
         create_pybuild(file)
-        __safe_load_file(TMP_FILE)
+        _safe_load_file(TMP_FILE)
 
 
 def test_module_write():
@@ -344,6 +343,8 @@ class Package(Pybuild1):
 
 def test_module_write_2():
     """Tests that module subattribute changes do not propagate from within pybuilds"""
+    import pybuild
+
     file = """
 import pybuild
 from pybuild import Pybuild1
@@ -371,9 +372,9 @@ class Package(Pybuild1):
     DESC="Test"
     LICENSE="GPL-3"
 """
-    with pytest.raises(SyntaxError):
+    with pytest.raises(ImportError):
         create_pybuild(file)
-        __safe_load_file(TMP_FILE)
+        _safe_load_file(TMP_FILE)
 
 
 @pytest.mark.xfail(reason="Portability feature not implemented")
@@ -412,7 +413,7 @@ class Package(Pybuild1):
 """
     with pytest.raises(SyntaxError):
         create_pybuild(file)
-        __safe_load_file(TMP_FILE)
+        _safe_load_file(TMP_FILE)
 
 
 def test_getattr():
@@ -865,7 +866,7 @@ class Package(Pybuild1):
         foo >> 1
 """
     create_pybuild(file)
-    __safe_load_file(TMP_FILE)
+    _safe_load_file(TMP_FILE)
 
 
 def test_augattribute():
@@ -884,7 +885,7 @@ class Package(Pybuild1):
         self.foo += 10
 """
     create_pybuild(file)
-    __safe_load_file(TMP_FILE)
+    _safe_load_file(TMP_FILE)
 
 
 def test_path():
@@ -974,3 +975,29 @@ class Package(Pybuild1):
 
     pkg = load_file(TMP_FILE)
     src_prepare(pkg, env.TMP_DIR)
+
+
+def test_import_common():
+    """Tests that importing from common directly fails"""
+    file = """
+import common
+
+class Package(Pybuild1):
+    pass
+"""
+    create_pybuild(file)
+    with pytest.raises(RuntimeError):
+        _safe_load_file(TMP_FILE)
+
+
+def test_global_inheritance():
+    """Tests that inheriting from banned globals fails"""
+    file = """
+from types import SimpleNamespace
+
+class Package(SimpleNamespace):
+    pass
+"""
+    create_pybuild(file)
+    with pytest.raises(TypeError):
+        _safe_load_file(TMP_FILE)
