@@ -26,7 +26,7 @@ def setup_repo():
 def test_masters_esp():
     """Tests that we detect esp masters properly"""
 
-    configure(["test/quill-of-feyfolken-2.0.2"])
+    configure(["=test/quill-of-feyfolken-2.0.2"])
     path = os.path.join(
         env.prefix().ROOT, "pkg", "test", "quill-of-feyfolken", "Quill of Feyfolken.esp"
     )
