@@ -27,7 +27,7 @@ def setup():
 
 def test_module(setup):
     """Tests that modules work as expected"""
-    configure(["test/test-module"], no_confirm=True)
+    configure(["test/test-module"])
     assert os.path.exists(
         os.path.join(env.prefix().CONFIG_PROTECT_DIR, "foo.cfg_protect")
     )
@@ -36,6 +36,6 @@ def test_module(setup):
 
 def test_module_cli(setup):
     """Tests that the CLI module interface works"""
-    configure(["test/test-module"], no_confirm=True)
+    configure(["test/test-module"])
     sys.argv = ["portmod", "test", "select", "test-module", "list"]
     main()
